﻿// -----------------------------------------------------------------------
// <copyright file="BusinessHelper.cs" company="Evolis Card Printer">
//     Copyright (c) Evolis Card Printer. All rights reserved.
// </copyright>
// -----------------------------------------------------------------------

namespace ClientRuntime
{
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using System.IO;
    using System.Linq;
    using System.Text;
    using System.Threading.Tasks;


    // 
    // --------------------------------------------------------------------------------
    // 
    public static class BusinessHelper
    {
        public static string IP;
        public static string Port;
        public static string CommType;
        public static string Printer;
        public static string Model = "Evolis Primacy 2";
        public static Color FgColor = Color.Black;
        public static Color BgColor = Color.WhiteSmoke;

        // 
        // ----------------------------------------------------------------------------
        // 
        public static EspfCommunicationPocessType CommProcType
        {
            get
            {
                return (CommType == "TCP") ?
                    EspfCommunicationPocessType.TCP :
                    EspfCommunicationPocessType.PIPE;
            }
        }


        // 
        // ----------------------------------------------------------------------------
        // 
        public static string PrinterSetEvent(string sEvent, string printer)
        {
            if (string.IsNullOrEmpty(sEvent) || string.IsNullOrEmpty(printer))
                throw new NullReferenceException("BusinessHelper.PrinterSetEvent");

            string id = "225";
            EspfRequest.EspfParamsSupervision p = new EspfRequest.EspfParamsSupervision() { action = sEvent, device = printer };
            
            string result = EspfClientProcessor.Send(
                IP, Port, CommProcType,
                new EspfRequest.EspfSupervision() { method = "SUPERVISION.SetEvent", id = id, parameters = p });
            EspfRequest.EspfResponse answer = new EspfRequest.EspfResponse();
            answer.result = result;
            return result;
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        public static string PrinterGetEvent(string printer)
        {
            if (string.IsNullOrEmpty(printer))
                throw new NullReferenceException("BusinessHelper.PrinterGetEvent");

            string id = "222";
            EspfRequest.EspfParamsSupervision p = new EspfRequest.EspfParamsSupervision() { device = printer };

            string result = EspfClientProcessor.Send(
                IP, Port, CommProcType,
                new EspfRequest.EspfSupervision() { method = "SUPERVISION.GetEvent", id = id, parameters = p });
            return result;
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        public static string PrinterGetState(string printer)
        {
            if (string.IsNullOrEmpty(printer))
                throw new NullReferenceException("BusinessHelper.PrinterGetState");

            string id = "223";
            EspfRequest.EspfParamsSupervision p = new EspfRequest.EspfParamsSupervision() { device = printer };

            string result = EspfClientProcessor.Send(
                IP, Port, CommProcType,
                new EspfRequest.EspfSupervision() { method = "SUPERVISION.GetState", id = id, parameters = p });

            return result;
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        public static string[] GetSupervisedPrinter(string model, string level)
        {
            string[] res;
            try
            {
                if (string.IsNullOrEmpty(model))
                    throw new NullReferenceException("BusinessHelper.GetSupervisedPrinter");

                string id = "220";
                EspfRequest.EspfParamsSupervision p = new EspfRequest.EspfParamsSupervision() { device = model, level = level };

                string result = EspfClientProcessor.Send(
                    IP, Port, CommProcType,
                    new EspfRequest.EspfSupervision() { method = "SUPERVISION.List", id = id, parameters = p });

                res = (result != "") ? result.Split(new char[] { ';' }) : null;
            }
            catch (Exception ex)
            {
                res = null;
            }
            return res;
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        public static string SendEcho(string echo)
        {
            string id = "ECHO10";
            EspfRequest.EspfParamsEcho p = new EspfRequest.EspfParamsEcho() { data = echo };

            string result = EspfClientProcessor.Send(
                IP, Port, CommProcType,
                new EspfRequest.EspfEcho() { method = "ECHO.Echo", id = id, parameters = p });

            return result;
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        public static string AddRemoveDevice(string device, bool needAdd)
        {
            string id = "SUPERVISION25";
            EspfRequest.EspfParamsSupervision p = new EspfRequest.EspfParamsSupervision() { device = device };

            string result = EspfClientProcessor.Send(
              IP, Port, CommProcType,
              new EspfRequest.EspfSupervision() { method = (needAdd) ? "SUPERVISION.AddDevice" : "SUPERVISION.RemoveDevice", id = id, parameters = p });

            return result;
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        public static string SendCommand(string device, string cmd, string timeout)
        {
            string id = "CMD42";
            EspfRequest.EspfParamsCmd p = new EspfRequest.EspfParamsCmd() { command = cmd, device = device, timeout = timeout };

            string result = EspfClientProcessor.Send(
                IP, Port, CommProcType,
                new EspfRequest.EspfCmd() { method = "CMD.SendCommand", id = id, parameters = p });

            return result;
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        public static string GetStatus(string device)
        {
            string id = "CMD54";
            EspfRequest.EspfParamsCmd p = new EspfRequest.EspfParamsCmd() { device = device };

            string result = EspfClientProcessor.Send(
                IP, Port, CommProcType,
                new EspfRequest.EspfCmd() { method = "CMD.GetStatus", id = id, parameters = p });

            return result;
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        public static byte[] ReadFully(Stream stream, int initialLength)
        {
            // If we've been passed an unhelpful initial length, just
            // use 32K.
            if (initialLength < 1)
            {
                initialLength = 32768;
            }

            byte[] buffer = new byte[initialLength];
            int read = 0;

            int chunk;
            while ((chunk = stream.Read(buffer, read, buffer.Length - read)) > 0)
            {
                read += chunk;

                // If we've reached the end of our buffer, check to see if there's
                // any more information
                if (read == buffer.Length)
                {
                    int nextByte = stream.ReadByte();

                    // End of stream? If so, we're done
                    if (nextByte == -1)
                    {
                        return buffer;
                    }

                    // Nope. Resize the buffer, put in the byte we've just
                    // read, and continue
                    byte[] newBuffer = new byte[buffer.Length * 2];
                    Array.Copy(buffer, newBuffer, buffer.Length);
                    newBuffer[read] = (byte)nextByte;
                    buffer = newBuffer;
                    read++;
                }
            }
            // Buffer is now too big. Shrink it.
            byte[] ret = new byte[read];
            Array.Copy(buffer, ret, read);
            return ret;
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        public static string Truncate(this string value, int maxLength)
        {
            if (string.IsNullOrEmpty(value)) return value;
            return value.Length <= maxLength ? value : value.Substring(0, maxLength);
        }

    }
}
