﻿// -----------------------------------------------------------------------
// <copyright file="EpsfClientProcessor.cs" company="Evolis Card Printer">
//     Copyright (c) Evolis Card Printer. All rights reserved.
// </copyright>
// -----------------------------------------------------------------------
namespace ClientRuntime
{


    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Net.Sockets;
    using System.Text;
    using System.Threading;
    using System.Threading.Tasks;
    using System.IO.Pipes;                  // NamedPipeClientStream
    using ClientRuntime.EspfRequest;

    // 
    // --------------------------------------------------------------------------------
    // 
    public enum EspfCommunicationPocessType { TCP, PIPE, UNKNOWN };

    //
    // -----------------------------------------------------------------------------------------
    //
    public static class EspfClientProcessor
    {
        public static string LastRequest = "";
        public static string LastAnswer = "";

        //
        // -------------------------------------------------------------------------------------
        //
        static public string Send(string ip, string port, EspfCommunicationPocessType ctype, object obj)
        {
            try
            {
                LastRequest = SerializationTools.jsonSerialize(obj, obj.GetType());
                LastRequest = LastRequest.Replace("parameters", "params");
            }
            catch (Exception e)
            {
                throw new Exception("Serialisation Error(Server Side) : " + e.Message + "\r\n obj:>" + ((EspfRequest.EspfRequest)obj).method);
            }

            LastAnswer = SendRequest(ctype, ip, port, LastRequest);
            EspfResponse ret = new EspfResponse();

            try
            {
                ret = (EspfResponse)SerializationTools.jsonDeserialize(LastAnswer, typeof(EspfResponse));
            }
            catch (Exception e)
            {
                throw new Exception("Serialisation Error : " + e.Message + "\r\n answer:>" + LastAnswer + "<\r\n");
            }

            if (ret.result == null)
            {
                return ret.error.message;
            }
            else
            {
                return ret.result;
            }
        }

        //
        // -------------------------------------------------------------------------------------
        //
        public static string SendRequest(EspfCommunicationPocessType ctype, string ip, string port, string request)
        {
            request = request.Replace("parameters", "params");

            switch (ctype)
            {
                case EspfCommunicationPocessType.TCP:
                    return SendRequestTcp(ip, port, request);

                case EspfCommunicationPocessType.PIPE:
                    return SendRequestPipe(ip, port, request);

                default:
                    throw new Exception("Unknown Serialization Process");

            }
        }


        public static string SendRequestTcp(string ip, string port, string request)
        {
            TcpClient client = new TcpClient(ip, Int32.Parse(port));
            string answer = string.Empty;
           
            try
            {
                NetworkStream ns = client.GetStream();
                int received = 0;
                byte[] datain = Encoding.UTF8.GetBytes(request);
                ns.Write(datain, 0, datain.Length);
                ns.Flush();
                byte[] data = new byte[1024];
                int recv = 0;
                bool hasAnswers = false;

                while ((ns.DataAvailable) || (!hasAnswers))
                {
                    recv = ns.Read(data, 0, data.Length);
                    received += recv;
                    answer += Encoding.UTF8.GetString(data, 0, recv);
                    hasAnswers = true;
                    Thread.Sleep(10);
                }
                ns.Close();
            }
            catch (Exception ex)
            {

            }
            finally
            {
                client.Close();
            }

            return answer;
        }

        //
        // -------------------------------------------------------------------------------------
        //
        public static string SendRequestPipe(string ip, string port, string request)
        {
            NamedPipeClientStream objPipeClient;

            string answer = string.Empty;

            objPipeClient = new NamedPipeClientStream(ip, port, PipeDirection.InOut);
            try
            {
                // Connexion au named pipe ou attente de la disponibilité du named pipe
                objPipeClient.Connect(1000);     
            
                // Paramétrage de la connexion
                objPipeClient.ReadMode = PipeTransmissionMode.Message;               

                if (objPipeClient.IsConnected == true)
                {
                    // Send request
                    byte[] datain = Encoding.UTF8.GetBytes(request);
                    objPipeClient.Write(datain, 0, datain.Length);
                    
                    // Get answer
                    int recv = 0;
                    byte[] data = new byte[1024];
                                       
                    do
                    {
                        recv = objPipeClient.Read(data, 0, data.Length);
                        answer += Encoding.UTF8.GetString(data, 0, recv);                        
                    } while (!objPipeClient.IsMessageComplete);
                }
            }
            catch (Exception ex)
            {                      

            }
            finally
            {
                objPipeClient.Close();
                objPipeClient.Dispose();
            }

            return answer;
        }

        //
        // ----------------------------------------------------
        //
        private static Random random = new Random((int)DateTime.Now.Ticks);

        //
        // -------------------------------------------------------------------------------------
        //
        public static string RandomString(int size)
        {
            StringBuilder builder = new StringBuilder();
            char ch;
            for (int i = 0; i < size; i++)
            {
                ch = Convert.ToChar(Convert.ToInt32(Math.Floor(94 * random.NextDouble() + 32)));
                builder.Append(ch);
            }

            return builder.ToString();
        }
    }
}


