﻿// -----------------------------------------------------------------------
// <copyright file="ParametersControl.cs" company="Evolis Card Printer">
//     Copyright (c) Evolis Card Printer. All rights reserved.
// </copyright>
// -----------------------------------------------------------------------

namespace ParametersControl
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Drawing;
    using System.Data;
    using System.Linq;
    using System.Text;
    using System.Threading.Tasks;
    using System.Windows.Forms;
    using ClientRuntime;

    //
    // ------------------------------------------------------------------
    //
    public partial class ParametersControl: UserControl
    {
        public event ColorChangeHandler notifyColorChange;
        public EventArgs e = null;
        public delegate void ColorChangeHandler(ParametersControl pui, EventArgs e);
        public string IP { get { return textBox1.Text; } }
        public string Port { get { return textBox2.Text; } }
        public string CommType { get { return (string)comboBox1.SelectedValue; } }

        //
        // --------------------------------------------------------------
        //
        public ParametersControl()
        {
            InitializeComponent();

            comboBox1.SelectedIndex = 0;

            textBox1.Text = Parameters.Get("ip");
            textBox2.Text = Parameters.Get("port");
            comboBox1.Text = Parameters.Get("commtype");

            BusinessHelper.Model = Parameters.Get("model");
            BusinessHelper.IP = textBox1.Text;
            BusinessHelper.Port = textBox2.Text;
            BusinessHelper.CommType = comboBox1.Text;

            UpdateColor();
        }
        
        //
        // --------------------------------------------------------------
        //
        private void textBox1_Validated(object sender, EventArgs e)
        {
            BusinessHelper.IP = textBox1.Text;
        }

        //
        // --------------------------------------------------------------
        //
        private void textBox2_Validated(object sender, EventArgs e)
        {
            BusinessHelper.Port = textBox2.Text;
        }

        //
        // --------------------------------------------------------------
        //
        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            BusinessHelper.CommType = comboBox1.Text;
            if (BusinessHelper.CommType == "TCP")
            {
                label1.Text = "IP:";
                label2.Text = "Port:";
                textBox1.Text = Parameters.Get("ip");
                textBox2.Text = Parameters.Get("port");
            }
            else
            {
                label1.Text = "Pipe Root:";
                label2.Text = "Pipe Name:";
                textBox1.Text = Parameters.Get("pipeRoot");
                textBox2.Text = Parameters.Get("pipeName");
            }
            BusinessHelper.IP = textBox1.Text;
            BusinessHelper.Port = textBox2.Text;
        }

        //
        // --------------------------------------------------------------
        //
        private void button1_Click(object sender, EventArgs e)
        {
            ColorDialog cd = new ColorDialog();
            DialogResult result = cd.ShowDialog();
            if (result == DialogResult.OK)
            {
                // Set form background to the selected color.
                BusinessHelper.FgColor = cd.Color;
                UpdateColor();
            }
        }

        //
        // --------------------------------------------------------------
        //
        private void UpdateColor()
        {
            this.ForeColor = BusinessHelper.FgColor;
            this.BackColor = BusinessHelper.BgColor;
            this.button1.BackColor = this.ForeColor;
            this.textBox1.ForeColor = BusinessHelper.FgColor;
            this.textBox1.BackColor = BusinessHelper.BgColor;
            this.textBox2.ForeColor = BusinessHelper.FgColor;
            this.textBox2.BackColor = BusinessHelper.BgColor;
            this.comboBox1.ForeColor = BusinessHelper.FgColor;
            this.comboBox1.BackColor = BusinessHelper.BgColor;
            if (notifyColorChange != null)
            {
                notifyColorChange(this, e);
            }
        }

        //
        // --------------------------------------------------------------
        //
        private void button2_Click(object sender, EventArgs e)
        {
            ColorDialog cd = new ColorDialog();
            DialogResult result = cd.ShowDialog();
            if (result == DialogResult.OK)
            {
                BusinessHelper.BgColor = cd.Color;
                UpdateColor();
            }
        }       
    }
}
