﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ClientRuntime;
using ClientRuntime.EspfRequest;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;

namespace PrintDemo
{
    public partial class PrintDemo : UserControl
    {

        private bool isSimplex;
        private bool isColor;
        private string session;

        public PrintDemo()
        {
            InitializeComponent();
            UpdateColor();
            isSimplex = true;
            isColor = true;
            session = "";
        }

        private void UpdateColor()
        {
            this.BackColor = BusinessHelper.BgColor;
            this.ForeColor = BusinessHelper.FgColor;
            this.textBox1.BackColor = BusinessHelper.BgColor;
            this.textBox1.ForeColor = BusinessHelper.FgColor;
            this.textBox2.BackColor = BusinessHelper.BgColor;
            this.textBox2.ForeColor = BusinessHelper.FgColor;
        }

        private void PrintDemo_Paint(object sender, PaintEventArgs e)
        {
            UpdateColor();
        }

        private void pictureBox1_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            DialogResult result = ofd.ShowDialog();
            ofd.Filter = "All Files (*.*)|*.*";
            if (result == DialogResult.OK)
            {
                pictureBox1.Image =  Image.FromFile(ofd.FileName); 
            }
        }

        private void pictureBox2_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            DialogResult result = ofd.ShowDialog();
            ofd.Filter = "All Files (*.*)|*.*";
            if (result == DialogResult.OK)
            {
                pictureBox2.Image = Image.FromFile(ofd.FileName);
            }
        }

        private void pictureBox3_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            DialogResult result = ofd.ShowDialog();
            ofd.Filter = "All Files (*.*)|*.*";
            if (result == DialogResult.OK)
            {
                pictureBox3.Image = Image.FromFile(ofd.FileName);
            }
        }

        private void printBegin()
        {
            string id = "PRINT562";
            EspfParamsPrint p = new EspfParamsPrint() { device = BusinessHelper.Printer };

            this.session = EspfClientProcessor.Send(
                BusinessHelper.IP, BusinessHelper.Port, BusinessHelper.CommProcType,
                new EspfPrint() { method = "PRINT.Begin", id = id, parameters = p });
        }

        private static string Bmp2Base64(Bitmap bitmap)
        {
            System.IO.MemoryStream stream = new System.IO.MemoryStream();
            bitmap.Save(stream, System.Drawing.Imaging.ImageFormat.Bmp);
            byte[] imageBytes = stream.ToArray();
            return Convert.ToBase64String(imageBytes);
        }

        private string printSetBitmap(Bitmap bitmap, string faceStr, string bmptype)
        {
            string id = "PRINT845";
            bitmap.Save("demo_" + faceStr + "_" + bmptype + ".bmp", System.Drawing.Imaging.ImageFormat.Bmp);
            string base64 = Bmp2Base64(bitmap);
            EspfParamsPrint p = new EspfParamsPrint() { face = faceStr, panel = bmptype, data = "base64:" + base64, session = this.session };

            EspfClientProcessor.Send(
               BusinessHelper.IP, BusinessHelper.Port, BusinessHelper.CommProcType,
               new EspfPrint() { method = "PRINT.SetBitmap", id = id, parameters = p });

            return base64.Length.ToString();
        }

        private void btnPrint_Click(object sender, EventArgs e)
        {
            textBox1.Text = "Print request cannot be printed because it is running in a thread";
            textBox2.Text = "Print response cannot be printed because it is running in a thread";

            string id = "PRINT866";
            EspfParamsPrint p = new EspfParamsPrint() { session = this.session };

            System.Threading.Thread t1 = new System.Threading.Thread(
                delegate()
                {
                    try
                    {
                        EspfClientProcessor.Send(BusinessHelper.IP, BusinessHelper.Port, BusinessHelper.CommProcType,
                            new EspfPrint() { method = "PRINT.Print", id = id, parameters = p });
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show("Cannot send request !", "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    }   
                });
            t1.Start();
        }

        private void btnSetCS_Click(object sender, EventArgs e)
        {
            try
            {
                printBegin();
                textBox1.Text = "";
                textBox2.Text = "";
                string pdata = "BColorBrightness=VAL10;BColorContrast=VAL10;BHalftoning=THRESHOLD;BMonochromeContrast=VAL10;BOverlayContrast=VAL10;BOverlayManagement=FULLVARNISH;BPageRotate180=OFF;FColorBrightness=VAL10;FColorContrast=VAL10;FHalftoning=THRESHOLD;FMonochromeContrast=VAL10;FOverlayContrast=VAL10;FPageRotate180=OFF;Duplex=NONE;GInputTray=FEEDER;GOutputTray=HOPPER;GPipeDetection=OFF;GRejectBox=DEFAULTREJECT;GRibbonType=RC_YMCKO;GSmoothing=ADVSMOOTH;IGSendIQLA=OFF";

                string id = "PRINT224";
                EspfParamsPrint p = new EspfParamsPrint() { data = pdata, session = this.session };
            
                string result = "\r\n" + EspfClientProcessor.Send(
                    BusinessHelper.IP, BusinessHelper.Port, BusinessHelper.CommProcType,
                    new EspfPrint() { method = "PRINT.Set", id = id, parameters = p });

                textBox1.Text = EspfClientProcessor.LastRequest;
                textBox2.Text = EspfClientProcessor.LastAnswer;
                isSimplex = true;
                isColor = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Cannot send request !", "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
            }   
        }

        private void btnSetMS_Click(object sender, EventArgs e)
        {
            try
            {
                printBegin();
                textBox1.Text = "";
                textBox2.Text = "";
                string pdata = "BColorBrightness=VAL10;BColorContrast=VAL10;BHalftoning=THRESHOLD;BMonochromeContrast=VAL10;BOverlayContrast=VAL10;BOverlayManagement=FULLVARNISH;BPageRotate180=OFF;FColorBrightness=VAL10;FColorContrast=VAL10;FHalftoning=THRESHOLD;FMonochromeContrast=VAL10;FOverlayContrast=VAL10;FOverlayManagement=BMPVARNISH;FPageRotate180=OFF;Duplex=NONE;GInputTray=FEEDER;GOutputTray=HOPPER;GPipeDetection=OFF;GRejectBox=DEFAULTREJECT;GRibbonType=RM_KO;GSmoothing=ADVSMOOTH;IGSendIQLA=OFF";

                string id = "PRINT225";
                EspfParamsPrint p = new EspfParamsPrint() { data = pdata, session = this.session };

                string result = "\r\n" + EspfClientProcessor.Send(
                    BusinessHelper.IP, BusinessHelper.Port, BusinessHelper.CommProcType,
                    new EspfPrint() { method = "PRINT.Set", id = id, parameters = p });

                textBox1.Text = EspfClientProcessor.LastRequest;
                textBox2.Text = EspfClientProcessor.LastAnswer;
                isSimplex = true;
                isColor = false;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Cannot send request !", "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
            }   
        }

        private void btnSetCD_Click(object sender, EventArgs e)
        {
            try
            {
                printBegin();
                textBox1.Text = "";
                textBox2.Text = "";
                string pdata = "BColorBrightness=VAL10;BColorContrast=VAL10;BMonochromeContrast=VAL10;BOverlayContrast=VAL10;BPageRotate180=OFF;FColorBrightness=VAL10;FColorContrast=VAL10;FMonochromeContrast=VAL10;FOverlayContrast=VAL10;FPageRotate180=OFF;Duplex=HORIZONTAL;GDuplexType=DUPLEX_CC;GInputTray=FEEDER;GOutputTray=HOPPER;GPipeDetection=OFF;GRejectBox=DEFAULTREJECT;GRibbonType=RC_YMCKO;GSmoothing=ADVSMOOTH;IGSendIQLA=OFF";

                string id = "PRINT226";
                EspfParamsPrint p = new EspfParamsPrint() { data = pdata, session = this.session };
           
                string result = "\r\n" + EspfClientProcessor.Send(
                    BusinessHelper.IP, BusinessHelper.Port, BusinessHelper.CommProcType,
                    new EspfPrint() { method = "PRINT.Set", id = id, parameters = p });

                textBox1.Text = EspfClientProcessor.LastRequest;
                textBox2.Text = EspfClientProcessor.LastAnswer;
                isSimplex = false;
                isColor = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Cannot send request !", "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
            }   
        }

        private void btnMD_Click(object sender, EventArgs e)
        {
            try
            {
                printBegin();
                textBox1.Text = "";
                textBox2.Text = "";
                string pdata = "BColorBrightness=VAL10;BColorContrast=VAL10;BMonochromeContrast=VAL10;BOverlayContrast=VAL10;BOverlayManagement=BMPVARNISH;BPageRotate180=OFF;FColorBrightness=VAL10;FColorContrast=VAL10;FMonochromeContrast=VAL10;FOverlayContrast=VAL10;FOverlayManagement=BMPVARNISH;FPageRotate180=OFF;Duplex=HORIZONTAL;GDuplexType=DUPLEX_MM;GInputTray=FEEDER;GOutputTray=HOPPER;GPipeDetection=OFF;GRejectBox=DEFAULTREJECT;GRibbonType=RM_KO;GSmoothing=ADVSMOOTH;IGSendIQLA=OFF";

                string id = "PRINT227";
                EspfParamsPrint p = new EspfParamsPrint() { data = pdata, session = this.session };

                string result = "\r\n" + EspfClientProcessor.Send(
                    BusinessHelper.IP, BusinessHelper.Port, BusinessHelper.CommProcType,
                    new EspfPrint() { method = "PRINT.Set", id = id, parameters = p });

                textBox1.Text = EspfClientProcessor.LastRequest;
                textBox2.Text = EspfClientProcessor.LastAnswer;
                isSimplex = false;
                isColor = false;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Cannot send request !", "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
            }   
        }

        private void btnSetBitmap_Click(object sender, EventArgs e)
        {
            Bitmap colorBmp;
            string res;
            string inReq = "";
            string outAns = "";


            textBox1.Text = "processing ...";
            textBox2.Text = "";
            try
            {
                if (isColor)
                {
                    colorBmp = new Bitmap(pictureBox1.Image, new Size(1016, 648));
                    res = printSetBitmap(colorBmp.Clone(new Rectangle(0, 0, colorBmp.Width, colorBmp.Height), PixelFormat.Format24bppRgb), "front", "color");
                    inReq = "Length : " + res + "\r\n" + BusinessHelper.Truncate(EspfClientProcessor.LastRequest, 100);
                    outAns = EspfClientProcessor.LastAnswer;
                }

                colorBmp = new Bitmap(pictureBox2.Image, new Size(1016, 648));
                res = printSetBitmap(ConvertTo1Bpp(colorBmp.Clone(new Rectangle(0, 0, colorBmp.Width, colorBmp.Height), PixelFormat.Format24bppRgb)), "front", "resin");

                inReq += "\r\n" + "Length : " + res + "\r\n" + BusinessHelper.Truncate(EspfClientProcessor.LastRequest, 100);
                outAns += "\r\n" + EspfClientProcessor.LastAnswer;

                colorBmp = new Bitmap(pictureBox3.Image, new Size(1016, 648));
                res = printSetBitmap(ConvertTo1Bpp(colorBmp.Clone(new Rectangle(0, 0, colorBmp.Width, colorBmp.Height), PixelFormat.Format24bppRgb)), "front", "varnish");

                inReq += "\r\n" + "Length : " + res + "\r\n" + BusinessHelper.Truncate(EspfClientProcessor.LastRequest, 100);
                outAns += "\r\n" + EspfClientProcessor.LastAnswer;

                if (!isSimplex)
                {
                    if (isColor)
                    {
                        colorBmp = new Bitmap(pictureBox1.Image, new Size(1016, 648));
                        res = printSetBitmap(colorBmp.Clone(new Rectangle(0, 0, colorBmp.Width, colorBmp.Height), PixelFormat.Format24bppRgb), "back", "color");

                        inReq += "\r\n" + "Length : " + res + "\r\n" + BusinessHelper.Truncate(EspfClientProcessor.LastRequest, 100);
                        outAns += "\r\n" + EspfClientProcessor.LastAnswer;
                    }

                    colorBmp = new Bitmap(pictureBox2.Image, new Size(1016, 648));
                    res = printSetBitmap(ConvertTo1Bpp(colorBmp.Clone(new Rectangle(0, 0, colorBmp.Width, colorBmp.Height), PixelFormat.Format24bppRgb)), "back", "resin");

                    inReq += "\r\n" + "Length : " + res + "\r\n" + BusinessHelper.Truncate(EspfClientProcessor.LastRequest, 100);
                    outAns += "\r\n" + EspfClientProcessor.LastAnswer;

                    colorBmp = new Bitmap(pictureBox3.Image, new Size(1016, 648));
                    res = printSetBitmap(ConvertTo1Bpp(colorBmp.Clone(new Rectangle(0, 0, colorBmp.Width, colorBmp.Height), PixelFormat.Format24bppRgb)), "back", "varnish");

                    inReq += "\r\n" + "Length : " + res + "\r\n" + BusinessHelper.Truncate(EspfClientProcessor.LastRequest, 100);
                    outAns += "\r\n" + EspfClientProcessor.LastAnswer;
                }

                textBox1.Text = inReq;
                textBox2.Text = outAns;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Cannot send request !", "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
            } 
        }

        public Bitmap ConvertTo1Bpp(Bitmap mono)
        {
            Bitmap bmp = mono.Clone(new Rectangle(0, 0, mono.Width, mono.Height), PixelFormat.Format32bppArgb);

            try
            {
                //bmp.RotateFlip(RotateFlipType.RotateNoneFlipY);
                BitmapData bmdo = bmp.LockBits(new Rectangle(0, 0, bmp.Width, bmp.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
                Bitmap bm = new Bitmap(mono.Width, mono.Height, PixelFormat.Format1bppIndexed);
                BitmapData bmdn = bm.LockBits(new Rectangle(0, 0, bm.Width, bm.Height), ImageLockMode.ReadWrite, PixelFormat.Format1bppIndexed);

                //scan through the pixels Y by X
                int x, y;
                for (y = 0; y < bmp.Height; y++)
                {
                    for (x = 0; x < bmp.Width; x++)
                    {
                        //generate the address of the colour pixel
                        int index = y * bmdo.Stride + (x * 4);
                        //check its brightness
                        if (Color.FromArgb(Marshal.ReadByte(bmdo.Scan0, index + 2),
                                        Marshal.ReadByte(bmdo.Scan0, index + 1),
                                        Marshal.ReadByte(bmdo.Scan0, index)).GetBrightness() > 0.5f)
                            SetIndexedPixel(x, y, bmdn, true); //set it if its bright.
                    }
                }

                bmp.UnlockBits(bmdo);

                int length = Math.Abs(bmdn.Stride) * bmdn.Height;
                byte[] bytes = new byte[length];

                Marshal.Copy(bmdn.Scan0, bytes, 0, length);
                bm.UnlockBits(bmdn);
                
                return bm;
            }
            catch (Exception e)
            {
                return null;
            }

        }

        protected void SetIndexedPixel(int x, int y, BitmapData bmd, bool pixel)
        {
            int index = y * bmd.Stride + (x >> 3);
            byte p = Marshal.ReadByte(bmd.Scan0, index);
            byte mask = (byte)(0x80 >> (x & 0x7));
            if (pixel)
                p |= mask;
            else
                p &= (byte)(mask ^ 0xff);
            Marshal.WriteByte(bmd.Scan0, index, p);
        }

        private void button8_Click(object sender, EventArgs e)
        {
            textBox1.Text = "";
            textBox2.Text = "";
            
            string id = "PRINT142";
            EspfParamsPrint p = new EspfParamsPrint() { session = this.session };

            try
            {
                string result = EspfClientProcessor.Send(
                    BusinessHelper.IP, BusinessHelper.Port, BusinessHelper.CommProcType,
                    new EspfPrint() { method = "PRINT.End", id = id, parameters = p });

                textBox1.Text = EspfClientProcessor.LastRequest;
                textBox2.Text = EspfClientProcessor.LastAnswer;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Cannot send request !", "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
            }   
        }

    }
}
