﻿// -----------------------------------------------------------------------
// <copyright file="PrinterLineState.cs" company="Evolis Card Printer">
//     Copyright (c) Evolis Card Printer. All rights reserved.
// </copyright>
// -----------------------------------------------------------------------

namespace StateDemo
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Drawing;
    using System.Data;
    using System.Linq;
    using System.Text;
    using System.Threading.Tasks;
    using System.Windows.Forms;
    using ClientRuntime;

    // 
    // --------------------------------------------------------------------------------
    // 
    public partial class PrinterLineState : UserControl
    {
        private Label lblPrinter;
        private Label lblStatus;
        private PictureBox pixStatus;
        private Button btnRetry;
        private Button btnCancel;
        private Button btnOk;
        private TextBox txtBox;

        public event DeviceListChangeHandler notifyDeviceListChange;
        public EventArgs e = null;
        public delegate void DeviceListChangeHandler(PrinterLineState pls, EventArgs e);

        // 
        // ----------------------------------------------------------------------------
        // 
        public PrinterLineState(string printername, ref TextBox txtbx)
        {
            InitializeComponent();

            txtBox = txtbx;

            FlowLayoutPanel flowLayoutPanel2 = new FlowLayoutPanel();
            FlowLayoutPanel flowLayoutPanel3 = new FlowLayoutPanel();

            flowLayoutPanel3.FlowDirection = FlowDirection.LeftToRight;
            flowLayoutPanel2.FlowDirection = FlowDirection.TopDown;
            flowLayoutPanel1.FlowDirection = FlowDirection.LeftToRight;

            //DEBUG
            //flowLayoutPanel3.BackColor = Color.Green;
            //flowLayoutPanel2.BackColor = Color.Pink;
            //flowLayoutPanel1.BackColor = Color.Blue;

            lblPrinter = new Label();
            lblPrinter.AutoSize = true;
            lblPrinter.Text = printername;
            lblPrinter.Font = new System.Drawing.Font("Verdana", 10.0F, FontStyle.Bold);
            lblStatus = new Label();
            lblStatus.AutoSize = true;
            pixStatus = new PictureBox();
            pixStatus.ImageLocation = "feu_vert.png";
            //pixStatus.Height = 64;
            pixStatus.SizeMode = PictureBoxSizeMode.AutoSize;

            Button btnRemove = new Button();
            btnRemove.BackgroundImage = new Bitmap("remove.png");
            btnRemove.BackgroundImageLayout = ImageLayout.Zoom;
            btnRemove.FlatStyle = FlatStyle.Flat;
            btnRemove.ForeColor = BusinessHelper.BgColor;
            btnRemove.Height = 50;
            btnRemove.Width = 50;
            btnRemove.Click += btnRemove_Click;

            btnRetry = new Button();
            btnRetry.AutoSize = true;
            btnRetry.FlatStyle = FlatStyle.Flat;

            flowLayoutPanel3.Controls.Add(btnRetry);

            btnCancel = new Button();
            btnCancel.AutoSize = true;
            btnCancel.FlatStyle = FlatStyle.Flat;

            flowLayoutPanel3.Controls.Add(btnCancel);

            btnOk = new Button();
            btnOk.AutoSize = true;
            btnOk.FlatStyle = FlatStyle.Flat;

            flowLayoutPanel3.Controls.Add(btnOk);

            UpdateContent();

            btnOk.Click += btnOk_Click;
            btnCancel.Click += btnCancel_Click;
            btnRetry.Click += btnRetry_Click;

            flowLayoutPanel2.Height = this.Height;
            flowLayoutPanel2.Width = flowLayoutPanel1.Width - pixStatus.Width;

            flowLayoutPanel3.Height = btnCancel.Height + 10;
            flowLayoutPanel3.Width = flowLayoutPanel2.Width;

            flowLayoutPanel2.Controls.Add(lblPrinter);
            flowLayoutPanel2.Controls.Add(lblStatus);
            flowLayoutPanel2.Controls.Add(flowLayoutPanel3);

            flowLayoutPanel1.Controls.Add(btnRemove);
            flowLayoutPanel1.Controls.Add(pixStatus);
            flowLayoutPanel1.Controls.Add(flowLayoutPanel2);

        }

        // 
        // ----------------------------------------------------------------------------
        // 
        void btnRemove_Click(object sender, EventArgs e)
        {
            try
            {
                string res = BusinessHelper.AddRemoveDevice(lblPrinter.Text, false /*remove*/);
                txtBox.Text = "Request:\r\n" + EspfClientProcessor.LastRequest + "\r\n\r\nAnswer:\r\n" + EspfClientProcessor.LastAnswer;
                if (notifyDeviceListChange != null)
                {
                    notifyDeviceListChange(this, e);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Cannot send request !", "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
            }             
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        void btnRetry_Click(object sender, EventArgs e)
        {
            try
            {
                string res = BusinessHelper.PrinterSetEvent(btnRetry.Text, lblPrinter.Text);
                btnRetry.Text = res;
                btnRetry.Enabled = false;
                txtBox.Text = "Request:\r\n" + EspfClientProcessor.LastRequest + "\r\n\r\nAnswer:\r\n" + EspfClientProcessor.LastAnswer;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Cannot send request !", "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
            }             
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        void btnCancel_Click(object sender, EventArgs e)
        {
            try
            {
                string res = BusinessHelper.PrinterSetEvent(btnCancel.Text, lblPrinter.Text);
                btnCancel.Text = res;
                btnCancel.Enabled = false;
                txtBox.Text = "Request:\r\n" + EspfClientProcessor.LastRequest + "\r\n\r\nAnswer:\r\n" + EspfClientProcessor.LastAnswer;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Cannot send request !", "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
            }             
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        void btnOk_Click(object sender, EventArgs e)
        {
            try
            {
                string res = BusinessHelper.PrinterSetEvent(btnOk.Text, lblPrinter.Text);
                btnOk.Text = res;
                btnOk.Enabled = false;
                txtBox.Text = "Request:\r\n" + EspfClientProcessor.LastRequest + "\r\n\r\nAnswer:\r\n" + EspfClientProcessor.LastAnswer;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Cannot send request !", "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
            }             
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        private void UpdateColor()
        {
            this.BackColor = BusinessHelper.BgColor;
            this.ForeColor = BusinessHelper.FgColor;
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        private void PrinterLineState_Paint(object sender, PaintEventArgs e)
        {
            UpdateColor();
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        public void UpdateContent()
        {
            string states = BusinessHelper.PrinterGetState(lblPrinter.Text);

            //txtBox.Text += "Request:\r\n" + EpsfClientProcessor.LastRequest + "\r\n\r\nAnswer:\r\n" + EpsfClientProcessor.LastAnswer + "\r\n\r\n";

            string events = BusinessHelper.PrinterGetEvent(lblPrinter.Text);

            //txtBox.Text += "Request:\r\n" + EpsfClientProcessor.LastRequest + "\r\n\r\nAnswer:\r\n" + EpsfClientProcessor.LastAnswer;

            lblStatus.Text = states;
            RaZButton();

            if (states.Contains("READY"))
            {
                pixStatus.ImageLocation = "feu_vert.png";
            }
            else if (states.Contains("ERROR"))
            {
                pixStatus.ImageLocation = "feu_rouge.png";
            }
            else if (states.Contains("@") || states.Contains("PRINTER_NOT_SUPERVISED") || string.IsNullOrEmpty(states))
            {
                if (notifyDeviceListChange != null)
                {
                    notifyDeviceListChange(this, e);
                }
            }
            else
            {
                pixStatus.ImageLocation = "feu_orange.png";
            }

            if (!events.Contains("NONE"))
            {
                string[] eventActions = events.Split(':');
                if (eventActions.Length == 2)
                {
                    pixStatus.ImageLocation = "attention.png";
                    string[] actions = eventActions[1].Split(',');
                    foreach (string action in actions)
                    {
                        if (action == "OK")
                        {
                            btnOk.Text = eventActions[0] + ':' + action;
                            btnOk.Visible = true;
                            btnOk.Enabled = true;
                        }

                        if (action == "RETRY")
                        {
                            btnRetry.Text = eventActions[0] + ':' + action;
                            btnRetry.Visible = true;
                            btnRetry.Enabled = true;
                        }

                        if (action == "CANCEL")
                        {
                            btnCancel.Text = eventActions[0] + ':' + action;
                            btnCancel.Visible = true;
                            btnCancel.Enabled = true;
                        }
                    }
                }
            }                       
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        private void RaZButton()
        {
            btnOk.Visible = false;
            btnRetry.Visible = false;
            btnCancel.Visible = false;
            btnOk.Enabled = false;
            btnRetry.Enabled = false;
            btnCancel.Enabled = false;
        }
    }
}
