﻿// -----------------------------------------------------------------------
// <copyright file="StateDemo.cs" company="Evolis Card Printer">
//     Copyright (c) Evolis Card Printer. All rights reserved.
// </copyright>
// -----------------------------------------------------------------------

namespace StateDemo
{

    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using System.IO;
    using System.Windows.Forms;
    using ClientRuntime;

    // 
    // --------------------------------------------------------------------------------
    // 
    public partial class StateDemo : UserControl
    {
        private PrinterLineState[] lines;
        private TextBox TxtbxRequest;
        private FlowLayoutPanel flp2;
        private Button btnRefresh;
        private Button btnNew;
        private int recomposeUI;

        public Timer MyTimer;
        public Timer MyTimer2;

        // 
        // ----------------------------------------------------------------------------
        // 
        public StateDemo()
        {
            InitializeComponent();

            recomposeUI = 0;

            flp2 = new FlowLayoutPanel();
            flp2.FlowDirection = FlowDirection.LeftToRight;
            flp2.Width = 700;
            flp2.Height = 70;

            btnRefresh = new Button();
            btnRefresh.BackgroundImage = new Bitmap("refresh.png");
            btnRefresh.BackgroundImageLayout = ImageLayout.Zoom;
            btnRefresh.FlatStyle = FlatStyle.Flat;
            btnRefresh.Height = 50;
            btnRefresh.Width = 50;
            btnRefresh.ForeColor = BusinessHelper.BgColor;
            btnRefresh.Click += btnRefresh_Click;
            flp2.Controls.Add(btnRefresh);

            btnNew = new Button();
            btnNew.BackgroundImage = new Bitmap("plus.png");
            btnNew.BackgroundImageLayout = ImageLayout.Zoom;
            btnNew.FlatStyle = FlatStyle.Flat;
            btnNew.ForeColor = BusinessHelper.BgColor;
            btnNew.Height = 50;
            btnNew.Width = 50;
            btnNew.Click += btnNew_Click;
            flp2.Controls.Add(btnNew);

            label1.Text = "Driver:";

            comboBox1.BackColor = BusinessHelper.BgColor;
            comboBox1.ForeColor = BusinessHelper.FgColor;
            

            TxtbxRequest = new TextBox();
            TxtbxRequest.Multiline = true;
            TxtbxRequest.Height = 150;
            TxtbxRequest.Width = 680;
            TxtbxRequest.BackColor = BusinessHelper.BgColor;
            TxtbxRequest.ForeColor = BusinessHelper.FgColor;

            ComposeFlowLayout();

            MyTimer = new Timer();
            MyTimer.Interval = (1 * 1000);
            MyTimer.Tick += new EventHandler(MyTimer_Tick);

            MyTimer2 = new Timer();
            MyTimer2.Interval = (1 * 5000);
            MyTimer2.Tick += new EventHandler(MyTimer2_Tick);
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        private void MyTimer2_Tick(object sender, EventArgs e)
        {
            if (recomposeUI>0)
            {
                ComposeFlowLayout();
                recomposeUI--;
            }
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        void btnNew_Click(object sender, EventArgs e)
        {
            AddDeviceUI adui = new AddDeviceUI();
            adui.ShowDialog();
            if (!string.IsNullOrEmpty(adui.DeviceName))
            {
                BusinessHelper.AddRemoveDevice(adui.DeviceName, true);
                TxtbxRequest.Text = "Request:\r\n" + EspfClientProcessor.LastRequest + "\r\n\r\nAnswer:\r\n" + EspfClientProcessor.LastAnswer;
                recomposeUI = 2;
            }

        }

        // 
        // ----------------------------------------------------------------------------
        // 
        private void ComposeFlowLayout()
        {
            try
            {
                this.flowLayoutPanel1.Controls.Clear();
                lines = null;
                this.flowLayoutPanel1.BackColor = BusinessHelper.BgColor;

                string[] printerList = BusinessHelper.GetSupervisedPrinter(BusinessHelper.Model, "0");
                if (printerList != null)
                {
                    lines = new PrinterLineState[printerList.Length];
                    for (int i = 0; i < printerList.Length; i++)
                    {
                        if (!string.IsNullOrEmpty(printerList[i]))
                        {
                            lines[i] = new PrinterLineState(printerList[i], ref TxtbxRequest);
                            lines[i].notifyDeviceListChange += new PrinterLineState.DeviceListChangeHandler(OnDeviceListChangeChange);
                        }
                    }
                }
                flowLayoutPanel1.Controls.Add(flp2);
                flowLayoutPanel1.Controls.AddRange(lines);
                flowLayoutPanel1.Controls.Add(TxtbxRequest);
                
            }
            catch (Exception ex)
            {

            }            
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        private void OnDeviceListChangeChange(PrinterLineState pls, EventArgs e)
        {
            if (recomposeUI ==0) recomposeUI = 2;
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        void btnRefresh_Click(object sender, EventArgs e)
        {
            ComposeFlowLayout();
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        private void MyTimer_Tick(object sender, EventArgs e)
        {
            if (lines != null)
            {
                foreach (PrinterLineState line in lines)
                {
                    if (line != null) line.UpdateContent();
                }
            }
        }

        // 
        // ----------------------------------------------------------------------------
        // 
        private void StateDemo_Paint(object sender, PaintEventArgs e)
        {
            this.ForeColor = BusinessHelper.FgColor;
            this.BackColor = BusinessHelper.BgColor;
            this.flowLayoutPanel1.BackColor = BusinessHelper.BgColor;
            TxtbxRequest.BackColor = BusinessHelper.BgColor;
            TxtbxRequest.ForeColor = BusinessHelper.FgColor;
            btnRefresh.ForeColor = BusinessHelper.BgColor;
            btnNew.ForeColor = BusinessHelper.BgColor;
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            BusinessHelper.Model = comboBox1.Text;
        }

        private void label1_Click(object sender, EventArgs e)
        {

        }

        //
        // --------------------------------------------------------------
        //

        private void PrinterSelector_Load(object sender, EventArgs e)
        {
            FileInfo myFile = new FileInfo("printerlist.txt");
            if (myFile.Exists)
            {
                List<string> myList = new List<string>();

                StreamReader myStream = new StreamReader(myFile.FullName);
                do
                {
                    myList.Add(myStream.ReadLine());

                } while (!myStream.EndOfStream);

                this.comboBox1.Items.Clear();
                this.comboBox1.Items.AddRange(myList.ToArray());
                this.comboBox1.SelectedItem = BusinessHelper.Model;
            }
        }
    }
}
