//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
//  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
//  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.

#include "EspfRequestTool.h"
#include "IpRequestSender.h"
#include "FileBuffer.h"

//-------------------------------------------------------------------

int sendRequestIp(char* input, char* ip, char* port, char* output, unsigned long maxAnswerSize)
{

#ifdef WIN32
	WSADATA wsa;
#endif

	int sockfd, numbytes;
	struct sockaddr_in their_addr;
	int ret = MAIN_OK;
	size_t bufferSize;
	char* buffer;
	char* answer;
	
	DWORD dwBytesRet = 0;
	struct tcp_keepalive alive;


#ifdef WIN32
	WSAStartup(MAKEWORD(2, 2), &wsa);
#endif

	loadBufferFromFile(input, &buffer, &bufferSize);
	if ((buffer == NULL) || (bufferSize == 0))
	{
		/* error message send by loadbuffer ... */
		return MAIN_ERR_CRITICAL;
	}

	if ((sockfd = socket(PF_INET, SOCK_STREAM, 0)) == -1) {
		fprintf(stderr, "Error socket creation\n");
		free(buffer);
		return MAIN_ERR_CRITICAL;
	}

#ifdef WIN32
	// Set the keepalive values to avoid network disconnection under Windows when waiting for an answer on a socket for a time superior to 2mn
	alive.onoff = TRUE;
	alive.keepalivetime = 90000;
	alive.keepaliveinterval = 1000;
	
	if (WSAIoctl(sockfd, SIO_KEEPALIVE_VALS, &alive, sizeof(alive), NULL, 0, &dwBytesRet, NULL, NULL) == SOCKET_ERROR)
	{
		fprintf(stderr, "Error setting KEEPALIVE\n");
		free(buffer);
		closesocket(sockfd);

		WSACleanup();

		return MAIN_ERR_CRITICAL;
	}
#endif

	their_addr.sin_family = AF_INET;
#ifdef WIN32
	SOCKADDR_IN MyAddr;
	int MyAddrSize = sizeof(struct sockaddr_storage);
	int retval = WSAStringToAddressA(ip, AF_INET, NULL, (LPSOCKADDR)&MyAddr, &MyAddrSize);
	their_addr.sin_addr = MyAddr.sin_addr;
#else
	inet_pton(AF_INET, ip, &their_addr.sin_addr);
#endif
	their_addr.sin_port = htons(atoi(port));

	if (connect(sockfd, (struct sockaddr *)&their_addr, sizeof their_addr) == -1)
	{
		fprintf(stderr, "Error socket connection\n");
		free(buffer);
		closesocket(sockfd);

#ifdef WIN32
		WSACleanup();
#endif

		return MAIN_ERR_CRITICAL;
	}

	if (send(sockfd, buffer, bufferSize, 0) == -1) 
	{
		fprintf(stderr, "Error sending buffer\n");
		free(buffer);
		closesocket(sockfd);

#ifdef WIN32
		WSACleanup();
#endif

		return MAIN_ERR_CRITICAL;
	}

	shutdown(sockfd, 1);

	answer = (char*)malloc(maxAnswerSize);
	if (NULL == answer)
	{
		fprintf(stderr, "Error memory allocation\n");
		free(buffer);
		closesocket(sockfd);

#ifdef WIN32
		WSACleanup();
#endif

		return MAIN_ERR_CRITICAL;
	}
	
	if ((numbytes = recv(sockfd, answer, STR_DEFAULTANSWER_SZ, 0)) == -1)
	{
		fprintf(stderr, "Error socket answer\n");
		ret = MAIN_ERR_CRITICAL;
	}
	else if (saveBufferToFile(output, answer, numbytes))
	{
		/* error message send by saveBufferToFile ... */
		ret = MAIN_ERR_CRITICAL;
	}

	free(answer);
	free(buffer);
	closesocket(sockfd);

#ifdef WIN32
	WSACleanup();
#endif

	return ret;
}

//-------------------------------------------------------------------