//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
//  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
//  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.

#include "PipeRequestSender.h"
#include "EspfRequestTool.h"
#include "FileBuffer.h"

int sendRequestPipe(char* input, char* pipename, char* unused, char* output, unsigned long maxAnswerSize)
{
#ifdef UNIX 
	return sendRequestPipeUnix(input, pipename, unused, output, maxAnswerSize);
#else 
#ifdef WIN32
	return sendRequestPipeWin(input, pipename, unused, output, maxAnswerSize);
#endif /*WIN32*/
#endif /*UNIX*/
}

#ifdef UNIX
int sendRequestPipeUnix(char* input, char* pipename, char* unused, char* output, unsigned long maxAnswerSize)
{ 
	return MAIN_ERR_CRITICAL; // @TODO : NOT YET IMPLEMENTED
}

	// from :http://beej.us/guide/bgipc/output/html/singlepage/bgipc.html#fifos
    // see also : http://tldp.org/LDP/lpg/node7.html

	//char s[300];
	//int num, fd;

	//mknod(FIFO_NAME, S_IFIFO | 0666, 0);

	//printf("waiting for readers...\n");
	//fd = open(FIFO_NAME, O_WRONLY);
	//printf("got a reader--type some stuff\n");

	//while (gets(s), !feof(stdin)) {
	//	if ((num = write(fd, s, strlen(s))) == -1)
	//		perror("write");
	//	else
	//		printf("speak: wrote %d bytes\n", num);
	//}

	//char s[300];
	//int num, fd;

	//mknod(FIFO_NAME, S_IFIFO | 0666, 0);

	//printf("waiting for writers...\n");
	//fd = open(FIFO_NAME, O_RDONLY);
	//printf("got a writer\n");

	//do {
	//	if ((num = read(fd, s, 300)) == -1)
	//		perror("read");
	//	else {
	//		s[num] = '\0';
	//		printf("tick: read %d bytes: \"%s\"\n", num, s);
	//	}
	//} while (num > 0);
#endif /*UNIX*/

#ifdef WIN32

/* From : https://msdn.microsoft.com/en-us/library/windows/desktop/aa365789(v=vs.85).aspx */

int sendRequestPipeWin(char* input, char* pipename, char* unused, char* output, unsigned long maxAnswerSize)
{
	HANDLE hPipe;
	BOOL fSuccess;
	DWORD cbRead, dwMode;

	int ret = MAIN_OK;
	size_t bufferSize;
	char* buffer;
	char* answer;

	loadBufferFromFile(input, &buffer, &bufferSize);
	if ((buffer == NULL) || (bufferSize == 0))
	{
		/* error message send by loadbuffer ... */
		return MAIN_ERR_CRITICAL;
	}

	// Try to open a named pipe; wait for it, if necessary. 
	while (1)
	{
		hPipe = CreateFile(
			pipename,       // pipe name 
			GENERIC_READ |  // read and write access 
			GENERIC_WRITE,
			0,              // no sharing 
			NULL,           // default security attributes
			OPEN_EXISTING,  // opens existing pipe 
			0,              // default attributes 
			NULL);          // no template file 

		// Break if the pipe handle is valid. 
		if (hPipe != INVALID_HANDLE_VALUE)
			break;

		// Exit if an error other than ERROR_PIPE_BUSY occurs. 
		if (GetLastError() != ERROR_PIPE_BUSY)
		{
			fprintf(stderr, "Could not open pipe\n");
			free(buffer);
			return MAIN_ERR_CRITICAL;
		}

		// All pipe instances are busy, so wait for 5 seconds. 
		if (!WaitNamedPipe(pipename, 5000))
		{
			fprintf(stderr, "Could not open pipe\n");			
			free(buffer);
			return MAIN_ERR_CRITICAL;
		}
	}

	// The pipe connected; change to message-read mode. 
	dwMode = PIPE_READMODE_MESSAGE;
	fSuccess = SetNamedPipeHandleState(
		hPipe,    // pipe handle 
		&dwMode,  // new pipe mode 
		NULL,     // don't set maximum bytes 
		NULL);    // don't set maximum time 

	if (!fSuccess)
	{
		fprintf(stderr, "SetNamedPipeHandleState failed\n");		
		CloseHandle(hPipe);
		free(buffer);
		return MAIN_ERR_CRITICAL;
	}

	// Send a message to the pipe server and read the response. 	
	answer = (char*)malloc(maxAnswerSize);
	if (NULL == answer)
	{
		fprintf(stderr, "Error memory allocation\n");
		CloseHandle(hPipe);
		return MAIN_ERR_CRITICAL;
	}

	fSuccess = TransactNamedPipe(
		hPipe,                  // pipe handle 
		buffer,                 // message to server
		bufferSize,             // message length 
		answer,                 // buffer to receive reply
		maxAnswerSize,          // size of read buffer
		&cbRead,                // bytes read
		NULL);                  // not overlapped 

	if (!fSuccess && (GetLastError() != ERROR_MORE_DATA))
	{
		fprintf(stderr, "TransactNamedPipe failed\n");
		CloseHandle(hPipe);
		free(answer);
		free(buffer);
		return MAIN_ERR_CRITICAL;
	}

	//while (1)
	//{
	//	_tprintf(TEXT("%s\n"), chReadBuf);

	//	// Break if TransactNamedPipe or ReadFile is successful
	//	if (fSuccess)
	//		break;

	//	// Read from the pipe if there is more data in the message.
	//	fSuccess = ReadFile(
	//		hPipe,      // pipe handle 
	//		chReadBuf,  // buffer to receive reply 
	//		BUFSIZE*sizeof(TCHAR),  // size of buffer 
	//		&cbRead,  // number of bytes read 
	//		NULL);    // not overlapped 

	//	// Exit if an error other than ERROR_MORE_DATA occurs.
	//	if (!fSuccess && (GetLastError() != ERROR_MORE_DATA))
	//		break;
	//	else _tprintf(TEXT("%s\n"), chReadBuf);
	//}

	CloseHandle(hPipe);
	free(buffer);

	ret = MAIN_OK;

	if (saveBufferToFile(output, answer, cbRead))
	{
		/* error message send by saveBufferToFile ... */
		ret = MAIN_ERR_CRITICAL;
	}

	free(answer);

	return ret; 
}
#endif /*WIN32*/
