/**
 * @file        EvoSdk1Lib1.h
 * @author      Evolis Software Team
 * @version     16/06/2020
 */

/*
    This file is part of the Evolis EVOSDK1.

    The Evolis EVOSDK1 is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The Evolis EVOSDK1 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with the Evolis EVOSDK1.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _EVOSDK1LIB1_H_
#define _EVOSDK1LIB1_H_

#ifdef EVOSDK1LIB1_EXPORTS
# define EVOSDK1LIB1_API __declspec(dllexport)
#else
# define EVOSDK1LIB1_API __declspec(dllimport)
#endif

#include "EvoSdk1Lib1Def.h"
#include <Windows.h>

//
// EvoSdk1Lib1 layer related
//
extern "C" EVOSDK1LIB1_API LPCSTR  EvoSdk1Lib1_GetVersion();
extern "C" EVOSDK1LIB1_API LPCWSTR EvoSdk1Lib1_GetVersionW();

//
// EPSDK - Global API
//
extern "C" EVOSDK1LIB1_API HANDLE EPSDK_GAPI_CreateCommunicationSettings(EPSDK_ConnectionType connectionType, const char* param1, const char* param2);
extern "C" EVOSDK1LIB1_API HANDLE EPSDK_GAPI_OpenPrinter(const char* printerName, HANDLE comSettingHandle);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_GAPI_ClosePrinter(HANDLE printerHandle);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_GAPI_DestroyCommunicationSettings(HANDLE comSettingHandle);

//
// EPSDK - SUPPORT TOOLS API
//
extern "C" EVOSDK1LIB1_API DWORD EPSDK_SupportTools_SendCommand(HANDLE printerHandle, const char* command, char** answer = NULL, int timeout = 30000);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_SupportTools_ResetCommunication(HANDLE printerHandle, DWORD timeout = 30000);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_SupportTools_GetDeviceBinaryStatus(HANDLE printerHandle, char** answer, DWORD timeout = 30000);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_SupportTools_PrintTestCard(HANDLE printerHandle, char** errorString = NULL, DWORD timeout = 60000);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_SupportTools_RunCleaningCycle(HANDLE printerHandle, char** errorString = NULL, DWORD timeout = 30000);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_SupportTools_MoveCard(HANDLE printerHandle, EPSDK_CardDestination cardPosition, DWORD timeout = 30000);

//
//  EPSDK - PRINTER SETTINGS API
//
extern "C" EVOSDK1LIB1_API DWORD EPSDK_PrinterSettings_SetCardInsertion(HANDLE printerHandle, EPSDK_InputTray value, DWORD timeout = 30000);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_PrinterSettings_SetCardEjection(HANDLE printerHandle, EPSDK_OutputTray value, DWORD timeout = 30000);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_PrinterSettings_SetCardEjectionOnError(HANDLE printerHandle, EPSDK_RejectBox value, DWORD timeout = 30000);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_PrinterSettings_SetCardPreloading(HANDLE printerHandle, bool preloadCard, DWORD timeout = 30000);

extern "C" EVOSDK1LIB1_API DWORD EPSDK_PrinterSettings_SetCardRetainingLength(HANDLE printerHandle, DWORD lengthValue, DWORD timeout = 30000);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_PrinterSettings_SetCardRetainingDelayBeforeAction(HANDLE printerHandle, DWORD lengthValue, DWORD timeout = 30000);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_PrinterSettings_SetCardRetainingAction(HANDLE printerHandle, EPSDK_BezelAction value, DWORD timeout = 30000);

//
// EPSDK - MAG ENCODING API
//
extern "C" EVOSDK1LIB1_API DWORD EPSDK_MagEncoding_ReadTrackData(HANDLE printerHandle, int trackNumber, char** outReadValue, DWORD timeout = 30000);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_MagEncoding_SetIsoNorm(HANDLE printerHandle, int trackNumber, EPSDK_IsoNorm iso, DWORD timeout = 5000);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_MagEncoding_SetTrackData(HANDLE printerHandle, int trackNumber, const char* dataToEncode, DWORD timeout = 5000);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_MagEncoding_WriteTrackData(HANDLE printerHandle, DWORD timeout = 30000);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_MagEncoding_SetCoercivity(HANDLE printerHandle, EPSDK_Coercivity coercivity, DWORD timeout = 5000);

//
//  EPSDK - PRINTER & RIBBON INFOS API
//
extern "C" EVOSDK1LIB1_API DWORD EPSDK_Infos_GetPrinterInfos(HANDLE printerHandle, EPSDK_PrinterInfo printerInfo, char** answer = NULL, DWORD timeout = 30000);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_Infos_GetRibbonInfos(HANDLE printerHandle, EPSDK_RibbonInfo ribbonInfo, char** answer = NULL, DWORD timeout = 30000);

//
// EPSDK - SUPERVISION API
//
extern "C" EVOSDK1LIB1_API DWORD EPSDK_Supervision_EnumEvolisSupervisedPrinters(HANDLE comHandle, int level, const char* modelName, char** reply, DWORD timeout = 30000);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_Supervision_GetDeviceState(HANDLE printerHandle, char** majorStatus, char** minorStatus);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_Supervision_AddDevice(HANDLE printerHandle);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_Supervision_RemoveDevice(HANDLE printerHandle);

//
//  EPSDK - PRINT API
//
extern "C" EVOSDK1LIB1_API DWORD EPSDK_Print_Begin(HANDLE printerHandle, char** sessionId, const char* customSessionId = NULL);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_Print_GetJobID(HANDLE printerHandle, char** sessionId);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_Print_Set(HANDLE printerHandle, const char* sessionId, const char* data);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_Print_SetBitmap(HANDLE printerHandle, EPSDK_FaceType face, EPSDK_PanelType panel, const char* sessionId, const char* data);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_Print_Exec(HANDLE printerHandle, const char* sessionId);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_Print_End(HANDLE printerHandle, const char* sessionId);

extern "C" EVOSDK1LIB1_API DWORD EPSDK_Print_GetEvent(HANDLE printerHandle, char** minorStatus, char** actionList);
extern "C" EVOSDK1LIB1_API DWORD EPSDK_Print_SetEvent(HANDLE printerHandle, const char* minorStatus, const char* action);

#endif /* _EVOSDK1LIB1_H_ */
