/**
 * @file
 * @author        Evolis Software Team
 * @version       28 July 2020
 */

/*
    This file is part of the Evolis EVOSDK1.

    The Evolis EVOSDK1 is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The Evolis EVOSDK1 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with the Evolis EVOSDK1.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _EVOSDK1LIB1DEF_H_
#define _EVOSDK1LIB1DEF_H_

///////////////////////////////////////////////////////////////////////////////
// EPSDK - Definitions
///////////////////////////////////////////////////////////////////////////////


enum EPSDK_ConnectionType {
    EPSDK_CT_Unknown,
    EPSDK_CT_Pipe,
    EPSDK_CT_IP,
};

enum EPSDK_FaceType {
    EPSDK_FT_Front,
    EPSDK_FT_Back,
};

enum EPSDK_PanelType {
    EPSDK_PT_Color,
    EPSDK_PT_Resin,
    EPSDK_PT_Varnish,
};

enum EPSDK_IsoNorm {
    EPSDK_IN_Iso1 = 1,
    EPSDK_IN_Iso2 = 2,
    EPSDK_IN_Iso3 = 3,
};

enum EPSDK_Coercivity {
    EPSDK_CO_Hico = 'h',
    EPSDK_CO_Loco = 'l',
    EPSDK_CO_Auto = 'a',
};

enum EPSDK_CardDestination {
    EPSDK_CD_PrintPos,
    EPSDK_CD_BackPos,
    EPSDK_CD_SmartPos,
    EPSDK_CD_ContactLessPos,
    EPSDK_CD_EjectCard,
    EPSDK_CD_RejectCard,
    EPSDK_CD_InsertEjectCard,
};

enum EPSDK_InputTray {
    EPSDK_IT_Feeder,
    EPSDK_IT_Auto,
    EPSDK_IT_Manual,
    EPSDK_IT_Bezel,
};
enum EPSDK_OutputTray {
    EPSDK_OT_Hopper,
    EPSDK_OT_Rear,
    EPSDK_OT_Bezel,
    EPSDK_OT_Manual,
    EPSDK_OT_LowerSlot,
};
enum EPSDK_RejectBox {
    EPSDK_RB_Default,         // Fw cmd: Pcrm;D
    EPSDK_RB_Hopper,          // Fw cmd: Pcrm;R
    EPSDK_RB_Manual,          // Fw cmd: Pcrm;M
    EPSDK_RB_Bezel,           // Fw cmd: Pcrm;I
    EPSDK_RB_LowerSlot,       // Fw cmd: Pcrm;K
    EPSDK_RB_Locked,          // Fw cmd: Pcrm;L
};
enum EPSDK_BezelAction {
    EPSDK_BA_Rejected,
    EPSDK_BA_Inserted,
    EPSDK_BA_Nothing,
};
enum EPSDK_RibbonInfo {
    EPSDK_RI_Type,
    EPSDK_RI_Zone,
    EPSDK_RI_RemainingCapacity,
    EPSDK_RI_TotalCapacity,
    EPSDK_RI_BatchNumber,
    EPSDK_RI_ProductCode,
    EPSDK_RI_ManufacturedDate,
    EPSDK_RI_Description,
};
enum EPSDK_PrinterInfo {
    EPSDK_PI_Fw,
    EPSDK_PI_Name,
    EPSDK_PI_Zone,
    EPSDK_PI_Model,
    EPSDK_PI_Duplex,
    EPSDK_PI_SmartOffset,
    EPSDK_PI_RetainingLength,
    EPSDK_PI_RetainingDelay,
    EPSDK_PI_RetainingAction,
    EPSDK_PI_PrintedPanelsCount,
    EPSDK_PI_InsertedCardsCount,
    EPSDK_PI_OptionEthernet,
    EPSDK_PI_OptionWifi,
    EPSDK_PI_OptionMag,
    EPSDK_PI_OptionSmart,
    EPSDK_PI_OptionContactLess,
    EPSDK_PI_OptionFlip,
    EPSDK_PI_OptionLaminator,
};

enum EPSDK_ReturnCode {
    // No error
    EPSDK_NO_ERROR                                  = 0,

    // 1 -> 99: Errors concerning the SDK usage.
    EPSDK_INTERNAL_ERROR                            = 1,
    EPSDK_INVALID_HANDLE                            = 2,
    EPSDK_INVALID_PARAMETER                         = 3,
    EPSDK_TIMEOUT                                   = 4,
    EPSDK_ERR_NO_COMMUNICATION_WITH_ESPF_SERVICE    = 5,
    EPSDK_ERR_NO_COMMUNICATION_WITH_PRINTER         = 6,
    EPSDK_SEE_ERROR_STRING                          = 7,
    EPSDK_INVALID_CMD                               = 8,
    EPSDK_INVALID_CMD_PARAM                         = 9,

    // 100 -> 199: Mag encoding errors.
    EPSDK_ME_NOT_AVAILABLE                          = 100,
    EPSDK_ME_BLANK_TRACK                            = 101,
    EPSDK_ME_BAD_DATA_FORMAT                        = 102,
    EPSDK_ME_WRITE_ERROR                            = 103,
    EPSDK_ME_FEEDER_EMPTY                           = 104,

    // 1000 -> 1199: SUPERVISION service errors.
    EPSDK_SV_SET_ACTION_ERROR                       = 1001,
    EPSDK_SV_DEVICE_NOT_FOUND                       = 1100,
    EPSDK_SV_DEVICE_ALREADY_SUPERVISED              = 1101,
    EPSDK_SV_DEVICE_ALREADY_UNSUPERVISED            = 1102,
    EPSDK_SV_NO_PENDING_EVENT                       = 1103,
    EPSDK_SV_INVALID_ACTION                         = 1104,
    EPSDK_SV_INVALID_EVENT                          = 1105,

    // 1200 -> 1399: Errors concerning the communication between
    //               the library and the Evolis Premium SDK.
    EPSDK_CD_DEVICE_NOT_FOUND                       = 1302,
    EPSDK_CD_CANT_GET_STATUS                        = 1303,

    // 1600 -> 1799: PRINT service errors.
    EPSDK_PR_PRINT_ERROR                            = 1600,
    EPSDK_PR_SESSION_ALREADY_RESERVED               = 1700,
    EPSDK_PR_INVALID_PRINTING_SESSION               = 1701,
    EPSDK_PR_PRINT_ALREADY_IN_PROGRESS              = 1702,
    EPSDK_PR_SETTING_OR_BITMAP_MISSING              = 1703,
    EPSDK_PR_PROCESSING_ERROR                       = 1704,
    EPSDK_PR_JOB_CANCELED                           = 1705,
    EPSDK_PR_NO_PRINTING_SESSION                    = 1706,
};

#endif  // _EVOSDK1LIB1DEF_H_
