#include "demo.h"
#include "EvoSdk1Base.h"
#include "EvoSdk1Lib1.h"

#include <conio.h>
#include <cstdio>
#include <fstream>
#include <iostream>
#include <thread>

static void printSessionManagement(HANDLE ph, bool* endFlag) {
    char* minorState = NULL;
    char* actionList = NULL;
    int   seconds    = 0;
    DWORD ec;

    while (!*endFlag) {
        ec = EPSDK_Print_GetEvent(ph, &minorState, &actionList);
        printf(" >> EPSDK_Print_GetEvent() -> (%d) - alive for %d seconds\n", ec, seconds);
        printf(" -- minorState = %s\n", minorState);
        printf(" -- actionList = %s\n", actionList);
        if (ec == EPSDK_NO_ERROR && strcmp(minorState, "NONE") != 0) {
            std::string action;

            printf(" ?? Please type in your desired action: ");
            if (!(std::cin >> action)) {
                std::cin.clear();
                std::cin.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
            }

            ec = EPSDK_Print_SetEvent(ph, minorState, action.data());
            printf(" >> EPSDK_Print_SetEvent() -> %s (%d)\n",
                (ec == EPSDK_NO_ERROR) ? OK : KO, ec);
        }

        Sleep(3000);
        seconds += 3;
    }
}

static void demoPrint(Data* d, bool doDuplex)
{
    HANDLE comHandle;
    HANDLE printerHandle;

    if (demoOpen(d->serverPipe, d->printerName, &comHandle, &printerHandle)) {
        char* printingParams = (doDuplex)
            /* Duplex YMCKOK */ ? (char*) "GRibbonType=RC_YMCKOK;Duplex=HORIZONTAL;GDuplexType=DUPLEX_CM;FOverlayManagement=FULLVARNISH"
            /* Simplex YMCKO */ : (char*) "GRibbonType=RC_YMCKO;FOverlayManagement=FULLVARNISH";
        bool   endFlag = false;
        char*  reply;
        char*  fileData;
        size_t fileSize;
        DWORD  ec;

        printf("=== PRINT API ===\n");

        ec = EPSDK_Print_Begin(printerHandle, &reply);
        printf(" >> Begin print session.               %s (%d)\n",
            (ec == EPSDK_NO_ERROR) ? OK : KO, ec);

        if (ec == EPSDK_PR_SESSION_ALREADY_RESERVED) {
            if (!EPSDK_Print_GetJobID(printerHandle, &reply)) {
                char c;

                printf(" >> Session already reserved: %s\n", reply);
                printf(" ?? Do you want to end that session and start a new one (y/n) ?\n");
                if ((c = _getch()) == 'y' || c == 'Y') {
                    ec = EPSDK_Print_End(printerHandle, reply);
                    demoPrint(d, doDuplex);
                } else {
                    printf(" >> Aborting\n");
                }
            }
        } else if (ec != EPSDK_NO_ERROR) {
            printf(" >> Aborting\n");
        } else {
            // Backup the session id to be sure it's not overriden by
            // the library.
            char sessionId[128];
            strcpy(sessionId, reply);

            ec = EPSDK_Print_Set(printerHandle, sessionId, printingParams);
            printf(" >> Set printing parameters.           %s (%d)\n",
                (ec == EPSDK_NO_ERROR) ? OK : KO, ec);

            if (loadFile("resources/frontBitMap.bmp", &fileData, &fileSize)) {
                char*  b64Data;
                UINT b64Size;

                // It's mandatory to encode the file to base 64 before
                // sending it to the API.
                // First we get the encoded size:
                EvoSdk1Base_EncodeBinary((LPBYTE)fileData, fileSize, NULL, &b64Size);
                // Then we encode the file content:
                b64Data = new char[b64Size];
                EvoSdk1Base_EncodeBinary((LPBYTE)fileData, fileSize, &b64Data, &b64Size);

                ec = EPSDK_Print_SetBitmap(printerHandle, EPSDK_FT_Front, EPSDK_PT_Color, sessionId, b64Data);
                printf(" >> Setting EPSDK_FT_Front bitmap.     %s (%d)\n",
                    (ec == EPSDK_NO_ERROR) ? OK : KO, ec);

                // Don't forget to free the file content and the base64 encoded data.
                delete b64Data;
                delete fileData;
            }

            if (doDuplex && loadFile("resources/backBitMap.bmp", &fileData, &fileSize)) {
                char*  b64Data;
                UINT b64Size;

                // It's mandatory to encode the file to base 64 before
                // sending it to the API.
                // First we get the encoded size:
                EvoSdk1Base_EncodeBinary((LPBYTE)fileData, fileSize, NULL, &b64Size);
                // Then we encode the file content:
                b64Data = new char[b64Size];
                EvoSdk1Base_EncodeBinary((LPBYTE)fileData, fileSize, &b64Data, &b64Size);

                ec = EPSDK_Print_SetBitmap(printerHandle, EPSDK_FT_Back, EPSDK_PT_Resin, sessionId, b64Data);
                printf(" >> Setting EPSDK_FT_Back bitmap.      %s (%d)\n",
                    (ec == EPSDK_NO_ERROR) ? OK : KO, ec);

                // Don't forget to free the file content and the base64 encoded data.
                delete b64Data;
                delete fileData;
            }

            // The printing process must be launched from the current thread.
            // The thread will be blocked until the printing was done.
            //
            // To receive events from the printer during the print process, we
            // have to start a new thread that will be in charge of polling the
            // printer for events.
            std::thread t1(printSessionManagement, printerHandle, &endFlag);

            printf(" >> EPSDK_Print_Print() started.       %s (%d)\n",
                (ec == EPSDK_NO_ERROR) ? OK : KO, ec);
            ec = EPSDK_Print_Exec(printerHandle, sessionId);
            printf(" >> EPSDK_Print_Print() end.           %s (%d)\n",
                (ec == EPSDK_NO_ERROR) ? OK : KO, ec);
            endFlag = true;

            ec = EPSDK_Print_End(printerHandle, sessionId);
            printf(" >> Ending print session.              %s (%d)\n",
                (ec == EPSDK_NO_ERROR) ? OK : KO, ec);

            t1.join();
        }
        demoClose(comHandle, printerHandle);
    }
}

void demoPrintSimplex(Data* data)
{
    demoPrint(data, false);
}

void demoPrintDuplex(Data* data)
{
    demoPrint(data, true);
}
