#include "demo.h"
#include "EvoSdk1Lib1.h"

#include <cstdio>

void demoSupervision(Data* d)
{
    HANDLE comHandle;
    HANDLE printerHandle;

    if (demoOpen(d->serverPipe, d->printerName, &comHandle, &printerHandle)) {
        char* reply;
        DWORD errorCode;

        printf("=== SUPERVISION API ===\n");
        printf(" >> Enumerate devices with major and minor states:\n");
        errorCode = EPSDK_Supervision_EnumEvolisSupervisedPrinters(comHandle, 2, d->modelName, &reply);
        if (errorCode != EPSDK_NO_ERROR)
            printf(" -- Error: EPSDK_Supervision_EnumEvolisSupervisedPrinters() returned with code '%d'.\n", errorCode);
        else
            printf(" -- %s\n", reply);

        printf(" >> Enumerate devices without any state:\n");
        errorCode = EPSDK_Supervision_EnumEvolisSupervisedPrinters(comHandle, 0, d->modelName, &reply);
        if (errorCode != EPSDK_NO_ERROR) {
            printf(" -- Error: EPSDK_Supervision_EnumEvolisSupervisedPrinters() returned with code '%d'.\n", errorCode);
        }
        else {
            auto printers = split(reply, ';');

            printf(" -- %s\n", reply);
            printf(" >> Getting states with EPSDK_Supervision_GetDeviceState():\n");
            for (auto printer : printers) {
                char*  majorStatus;
                char*  minorStatus;

                printf(" -- %s:\n", printer.data());
                errorCode = EPSDK_Supervision_GetDeviceState(printerHandle, &majorStatus, &minorStatus);
                if (errorCode != EPSDK_NO_ERROR) {
                    printf("    Error: EPSDK_Supervision_GetDeviceState() returned with code '%d'.\n", errorCode);
                }
                else {
                    printf("    Device     : '%s'\n", d->printerName);
                    printf("    majorStatus: '%s'\n", majorStatus);
                    printf("    minorStatus: '%s'\n\n", minorStatus);
                }
            }
        }

        printf(" >> Remove devices from supervision(please wait 5 sec.)\n");
        errorCode = EPSDK_Supervision_RemoveDevice(printerHandle);
        if (errorCode != EPSDK_NO_ERROR) {
            printf("  --Error: EPSDK_Supervision_RemoveDevice() returned with code '%d'.\n", errorCode);
        }
        else {
            Sleep(5000);
            char*  majorStatus;
            char*  minorStatus;

            errorCode = EPSDK_Supervision_GetDeviceState(printerHandle, &majorStatus, &minorStatus);
            if (errorCode != EPSDK_NO_ERROR) {
                printf("    Error: EPSDK_Supervision_GetDeviceState() returned with code '%d'.\n", errorCode);
            }
            else {
                printf("    Device     : '%s'\n", d->printerName);
                printf("    majorStatus: '%s'\n", majorStatus);
                printf("    minorStatus: '%s'\n\n", minorStatus);
            }
        }

        printf(" >> Add devices to supervision(please wait 10 sec.)\n");
        errorCode = EPSDK_Supervision_AddDevice(printerHandle);
        if (errorCode != EPSDK_NO_ERROR) {
            printf("  --Error: EPSDK_Supervision_AddDevice() returned with code '%d'.\n", errorCode);
        }
        else {
            Sleep(10000);
            char*  majorStatus;
            char*  minorStatus;

            errorCode = EPSDK_Supervision_GetDeviceState(printerHandle, &majorStatus, &minorStatus);
            if (errorCode != EPSDK_NO_ERROR) {
                printf("    Error: EPSDK_Supervision_GetDeviceState() returned with code '%d'.\n", errorCode);
            }
            else {
                printf("    Device     : '%s'\n", d->printerName);
                printf("    majorStatus: '%s'\n", majorStatus);
                printf("    minorStatus: '%s'\n\n", minorStatus);
            }
        }

        demoClose(comHandle, printerHandle);
    }
}

