#include "demo.h"
#include "EvoSdk1Lib1.h"

#include <cstdio>

void demoSupportTools(Data* d)
{
    HANDLE comHandle;
    HANDLE printerHandle;
    char*  reply;
    DWORD  ec;

    if (demoOpen(d->serverPipe, d->printerName, &comHandle, &printerHandle)) {
        printf("=== SUPPORT TOOLS & PRINTER SETTINGS ===\n");
        printf(" >> Moving a card in the printer:\n");

        //demo of several printer settings and card movement        
        ec = EPSDK_PrinterSettings_SetCardInsertion(printerHandle, EPSDK_IT_Feeder);
        printf(" -- Set card insertion to INPUT_FEEDER.     %s (%d)\n",
            (ec == EPSDK_NO_ERROR) ? OK : KO, ec);

        //insert card in print position 
        ec = EPSDK_SupportTools_MoveCard(printerHandle, EPSDK_CD_PrintPos, 30000);
        printf(" -- Move card to PRINT_POSITION.            %s (%d)\n",
            (ec == EPSDK_NO_ERROR) ? OK : KO, ec);

        //move card to smart position
        ec = EPSDK_SupportTools_MoveCard(printerHandle, EPSDK_CD_SmartPos, 30000);
        printf(" -- Move card to SMART_POSITION.            %s (%d)\n",
            (ec == EPSDK_NO_ERROR) ? OK : KO, ec);
        //
        //here the developper can use smart constructor SDK to perform ATR
        //operations, encode data on card etc...
        //

        //move card to contactless position
        ec = EPSDK_SupportTools_MoveCard(printerHandle, EPSDK_CD_ContactLessPos, 30000);
        printf(" -- Move card to CONTACTLESS_POSITION.      %s (%d)\n",
            (ec == EPSDK_NO_ERROR) ? OK : KO, ec);

        //set card ejection to hopper
        ec = EPSDK_PrinterSettings_SetCardEjection(printerHandle, EPSDK_OT_Hopper);
        printf(" -- Set card ejection to OUTPUT_HOPPER.     %s (%d)\n",
            (ec == EPSDK_NO_ERROR) ? OK : KO, ec);

        //Eject card
        ec = EPSDK_SupportTools_MoveCard(printerHandle, EPSDK_CD_EjectCard, 30000);
        printf(" -- Move card to EJECT_CARD.                %s (%d)\n",
            (ec == EPSDK_NO_ERROR) ? OK : KO, ec);



        //Card preloading is only active when using monochrome ribbon.
        //Can be set but the card will only be inserted after performing
        //a monochrome printing.
        ec = EPSDK_PrinterSettings_SetCardPreloading(printerHandle, true);
        printf(" >> Set preloading to TRUE.                 %s (%d)\n",
            (ec == EPSDK_NO_ERROR) ? OK : KO, ec);
        //Card retaining options and settings are only available using KC model
        //Today demo we are not using a KC model for practical reason, but be
        //assured that these methods has been implemented and tested.



        //Send a command to the printer with EPSDK_SupportTools_SendCommand().
        //We are sending a 'Rfv' command to get the firmware version.
        ec = EPSDK_SupportTools_SendCommand(printerHandle, "Rfv", &reply);
        printf(" >> Send a 'Rfv' command to the printer.    %s (%d)\n",
            (ec == EPSDK_NO_ERROR) ? OK : KO, ec);
        printf(" -- reply = %s\n", reply);

        //Read binary status of printer using GetDeviceBinaryStatus method
        //
        //The binary status is base64 encoded. It must be decoded to binary before use.
        //The EvoSdk1Base_DecodeBinary() method can be use to proceed.
        //
        //Full description of the binary data format can be found in the Premium SDK
        //Reference guide and Firmware documentation.
        ec = EPSDK_SupportTools_GetDeviceBinaryStatus(printerHandle, &reply);
        printf(" >> Read binary status.                     %s (%d)\n",
            (EPSDK_NO_ERROR == ec) ? OK : KO, ec);
        printf(" -- reply = %s\n", reply);
        demoClose(comHandle, printerHandle);
    }
}
