#include "demo.h"
#include "EvoSdk1Lib1.h"

#include <fstream>
#include <sstream>

/*!
 * Helper to open printer handles.
 * 
 * \param printerName The name of the printer to connect to.
 * \param sh A pointer to a communication settings HANDLE.
 * \param ph A pointer to a printer HANDLE.
 * \return True on success of false otherwise.
 */
bool demoOpen(const char* serverPipe, const char* printerName, HANDLE* sh, HANDLE* ph)
{
    if (sh != NULL) {
        *sh = EPSDK_GAPI_CreateCommunicationSettings(EPSDK_CT_Pipe, ".", serverPipe);
        if (*sh != NULL && ph != NULL) {
            *ph = EPSDK_GAPI_OpenPrinter(printerName, *sh);
            if (*ph == NULL) {
                EPSDK_GAPI_DestroyCommunicationSettings(*sh);
                *sh = NULL;
            }
        }
    }
    return (sh && *sh && (!ph || (ph && *ph)));
}

/*!
 * Helper to close the handles.
 * 
 * \param sh The communication handle to close.
 * \param ph The printer handle to close.
 */
void demoClose(HANDLE sh, HANDLE ph)
{
    if (ph != NULL)
        EPSDK_GAPI_ClosePrinter(ph);
    if (sh != NULL)
        EPSDK_GAPI_DestroyCommunicationSettings(sh);
}

/*!
 * Helper to split a string with a delimiter.
 * 
 * \param s The string to split.
 * \param delimiter The delimiter used to split the string s.
 * \return A std::vector containing string parts.
 */
std::vector<std::string> split(const std::string& s, char delimiter)
{
    std::vector<std::string>    tokens;
    std::string                 token;
    std::istringstream          stream(s);

    while (std::getline(stream, token, delimiter))
        tokens.push_back(token);
    return tokens;
}

/*!
 * Helper to load a file from disk.
 * 
 * \param filePath The path to the file to load.
 * \param data A pointer to a buffer that will receive file content.
 * \param size A pointer to store the file content size.
 * \return True on success or false otherwiser.
 */
bool loadFile(const char* filePath, char** data, size_t* size)
{
    std::ifstream ifs(filePath, std::ios::binary);

    if (ifs.is_open()) {
        auto buffer = std::vector<char>(
            std::istreambuf_iterator<char>(ifs),
            std::istreambuf_iterator<char>()
        );

        ifs.close();
        *size = buffer.size();
        *data = new char[*size];
        memcpy(*data, buffer.data(), *size);
        return true;
    }
    return false;
}
