﻿using DemoApp.Source_Files;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using static DemoApp.EVOSDK1Enums;
using static DemoApp.Source_Files.Helpers;

namespace DemoApp
{
    public class Program
    {
        
        static IntPtr pHandle = IntPtr.Zero;

        static void ChoosePrinter(ref Data d, string address, string pipe_port) {
            IntPtr handle = IntPtr.Zero;
            Console.WriteLine("=== TARGET PRINTER ===");

            handle = Helpers.CreateCommunication(address, pipe_port);

            if (null != handle) {
                IntPtr reply = IntPtr.Zero;
                int ec;
                int index = 0;

                ec = EVOSDK1Wrapper.EPSDK_Supervision_EnumEvolisSupervisedPrinters(handle, 0, d.ModelName, ref reply);
                // If more than one printer available then "reply" will contain
                // a string like "<PRINTER-0>;<PRINTER-1>;<PRINTER-N>" with
                // <PRINTER-X> replaced with the name of the printer.

                if ((int)EPSDK_ReturnCode.EPSDK_NO_ERROR == ec) {

                    string printers = Marshal.PtrToStringAnsi(reply).ToString();
                    List<string> printerList = printers.Split(';').ToList();

                    if (0 == printerList.Count || printers == "") {
                        Console.WriteLine(" >> No printer available for this model.");
                        d.PrinterAvailable = false;
                    } else if (1 == printerList.Count) {
                        Console.WriteLine(" >> Only one printer available.");
                        Console.WriteLine(" >> Selecting printer '{0}'", printerList.First().ToString());
                        d.PrinterAvailable = true;
                    } else {
                        int c = 0;
                        printerList.ForEach(x => {
                        Console.Write(c + " - "); WriteColored(x.ToString(), ConsoleColor.Magenta);
                            c++;
                        });
                        Console.Write("Please choose a printer number and press Enter : ");
                        index = Convert.ToInt32(Console.ReadLine());
                        d.PrinterAvailable = true;
                    }

                    d.PrinterName = printerList.ElementAt(index);
                }

                EVOSDK1Wrapper.EPSDK_GAPI_DestroyCommunicationSettings(handle);
            }
        }


        static void CheckPrinterStatus(string printerName, string address, string pipe_port, ref string majorStatus, ref string minorStatus)
        {
            IntPtr majorStatusPtr = default(IntPtr);
            IntPtr minorStatusPtr = default(IntPtr);

            IntPtr handle = CreateCommunication(address, pipe_port);

            if (null != handle)
            {
                IntPtr reply = IntPtr.Zero;
                IntPtr pHandle = IntPtr.Zero;
                int ec;

                pHandle = EVOSDK1Wrapper.EPSDK_GAPI_OpenPrinter(printerName, handle);
                ec = EVOSDK1Wrapper.EPSDK_Supervision_GetDeviceState(pHandle, ref majorStatusPtr, ref minorStatusPtr);

                if ((int)EPSDK_ReturnCode.EPSDK_NO_ERROR == ec)
                {
                    majorStatus = Marshal.PtrToStringAnsi(majorStatusPtr).ToString();
                    minorStatus = Marshal.PtrToStringAnsi(minorStatusPtr).ToString();
                }

                EVOSDK1Wrapper.EPSDK_GAPI_ClosePrinter(pHandle);
                EVOSDK1Wrapper.EPSDK_GAPI_DestroyCommunicationSettings(handle);
            }
        }



        /// <summary>
        /// Main for demo EvoSdk1Lib1
        /// </summary>
        /// <param name="args">Array of command input arguments </param>
        /// <returns>Error code</returns>
        static int Main(string[] args)
        {
            string printerName = "Evolis Primacy 2";
            string address = "127.0.0.1";
            string pipe_port = "Espf2Server00";

            var cpt = 0;

            foreach (string argv in args)
            {
                if (argv[0] == '-')
                {
                    if (argv.ToLower().StartsWith("-h") || argv.StartsWith("-?") || argv.ToLower().StartsWith("--help"))
                    {
                        DisplayHelp();
                        return 0;
                    }

                    if (argv.ToLower().StartsWith("-p") || argv.ToLower().StartsWith("--printertype"))
                    {
                        printerName = getParameterValue(args, cpt, argv);
                    }

                    if (argv.ToLower().StartsWith("-spi") || argv.ToLower().StartsWith("--serverpipe"))
                    {
                        pipe_port = getParameterValue(args, cpt, argv);
                    }
                }

                cpt++;
            }

            Data data = new Data(printerName);
            var commandsPrinterAvailable = new ICommand[]
                {
                    new demoInputModelName(),
                    new demoChoosePrinter(),
                    new demoPrinterInfos(),
                    new demoRibbonInfos(),
                    new demoMagEncoding(),
                    new demoPrintSimplex(),
                    new demoPrintDuplex(),
                    new demoSupervision(),
                    new demoSupportTools(),
                    new demoPrintTestCard(),
                    new demoCleaningCycle(),
                    new demoSendCommand(),
                    new ExitCommand()
                };
            var commandsPrinterUnavailable = new ICommand[]
                {
                    new demoInputModelName(),
                    new demoChoosePrinter(),
                    new ExitCommand()
                };
            
            ChoosePrinter(ref data, address, pipe_port);

            while (true) {
                var commands = data.PrinterAvailable ? commandsPrinterAvailable : commandsPrinterUnavailable;

                string majorStatus = null;
                string minorStatus = null;
                CheckPrinterStatus(data.PrinterName, address, pipe_port, ref majorStatus, ref minorStatus);

                Console.WriteLine("\n\n=== MENU ===\n");
                Console.WriteLine(" >> Server address       : {0}", address);
                Console.WriteLine(" >> Server pipe/port     : {0}", pipe_port);
                Console.WriteLine(" >> Printer model        : {0}", data.ModelName);
                Console.Write(" >> Printer name         : ");
                WriteColored(data.PrinterName, ConsoleColor.Magenta);
                Console.WriteLine(" >> Printer major status : " + majorStatus);
                Console.WriteLine(" >> Printer minor status : " + minorStatus);
                Console.WriteLine();

                for (int i = 0; i < commands.Length; i++) {
                    Console.WriteLine("    {0} - {1}", i, commands[i].Description);
                }

                var userChoice = string.Empty;
                var commandIndex = -1;

                do {
                    Console.WriteLine("\n?? Please, what is your option ? ");
                    userChoice = Console.ReadLine();
                }
                while (!int.TryParse(userChoice, out commandIndex) || commandIndex > commands.Length);

                commands[commandIndex].Execute(ref data, address, pipe_port);
            }
        }

        private static string getParameterValue(string[] args, int index, string parameter)
        {
            if ((args.Length > (index + 1)) && !args[index + 1].StartsWith("-"))
            {
                return args[index + 1];
            }
            else
            {
                throw new Exception("Error missing parameter value for parameter: " + parameter);
            }
        }

        /// <summary>
        /// Displays the help.
        /// </summary>
        private static void DisplayHelp()
        {
            Console.WriteLine("----------------------------");
            Console.WriteLine("List of parameters :");
            Console.WriteLine("----------------------------");
            Console.WriteLine("USAGE : -h  | -?  | --help        : Show this help section");
            Console.WriteLine("USAGE : -p        | --printertype : To select a printer type different than the default one(\"Evolis Primacy 2\").");
            Console.WriteLine("USAGE : -spi      | --serverpipe  : For printers related to the Evolis Premium Suite 2 (Primacy 2) do not specify any argument. It will use the default value: Espf2Server00. For Printers working with EPS 1 you must adapt the value and use the following one: ESPFServer00.");
        }

    }
}
