/*
    This file is part of the Evolis SDK.

    The Evolis SDK is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The Evolis SDK is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with the Evolis SDK.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "StdAfx.h"

#include "Bitmap.h"
#include "evolib.h"

#define	RgbToGray(r, g, b) (((long) (r)*74L + (long)(g)*155L +(long)(b)*27L) >> 8)

//extern BITMAPINFOHEADER	InfoHead;

////////////////////////////////////////////////////////////
//OPENBMP : BITMAP 24 bits or monochrome ONLY
/////////////////////////////////////////////////////////////
BOOL OpenBmp(char *pPathFile, unsigned char *pMem, long *dwSizeNeeded, BITMAPINFOHEADER	*InfoHead)
{
 FILE				*file;
 BITMAPFILEHEADER	*bmpFileHead;
 BITMAPINFOHEADER	*bmpInfoHead;
 
 unsigned int		WidthLine = 0;
 
 bmpFileHead= new BITMAPFILEHEADER;
 if(!bmpFileHead) return(FALSE);

 bmpInfoHead = new BITMAPINFOHEADER;
 if(!bmpInfoHead) {
	 delete(bmpFileHead);
	 *dwSizeNeeded = -1;
	 return(FALSE);
 }
 
 file = fopen( pPathFile, "rb" );
 if(!file) {
	delete(bmpInfoHead);
	delete(bmpFileHead);
	*dwSizeNeeded = -2;
	return(FALSE);
 }

 fread(bmpFileHead,sizeof(BYTE),sizeof(BITMAPFILEHEADER),file);
 fseek(file,sizeof(BITMAPFILEHEADER),SEEK_SET);
 fread(bmpInfoHead,sizeof(BYTE),sizeof(BITMAPINFOHEADER),file);
 memcpy(InfoHead,bmpInfoHead,sizeof(BITMAPINFOHEADER));

 if((InfoHead->biHeight == 1016 || InfoHead->biHeight == 648) && (InfoHead->biWidth == 648 || InfoHead->biWidth == 1016))
 {
	 if(dwSizeNeeded && !pMem) 
	 {
		if(bmpInfoHead->biBitCount == 24) //color
		{	
			*dwSizeNeeded = bmpInfoHead->biHeight * bmpInfoHead->biWidth * 5;
		}
		else
		{
			delete(bmpInfoHead);
			delete(bmpFileHead);
			fclose(file);
			*dwSizeNeeded = -3;
			return(FALSE);
		}
	 }
	 else
	 {
	  if(bmpInfoHead->biBitCount == 24 && bmpInfoHead->biPlanes == 1 )
	  {
		 if(pMem)
		 {
			ShareRVBtoMem(file, pMem, bmpInfoHead->biHeight, bmpInfoHead->biWidth,(UINT)(bmpInfoHead->biWidth * 3), bmpFileHead->bfOffBits);
		 }
	  }
	  else
	  {
			delete(bmpInfoHead);
			delete(bmpFileHead);
			fclose(file);
			*dwSizeNeeded = -4;
			return(FALSE);
	  }
	 }
	 delete(bmpInfoHead);
	 delete(bmpFileHead);
	 fclose(file);
	 return(TRUE);
 }
 delete(bmpInfoHead);
 delete(bmpFileHead);
 fclose(file);
 *dwSizeNeeded = -5;
 return(FALSE);
}
///////////////////////////////////////////////////////
// pMem ppV
//			 ___________
//			|           |
//			|           |
//			|   Yellow  |
//			|           |
//			|___________|
//			|           |
//			|           |
//			|   Magenta |
//			|           |
//			|___________|
//			|           |
//			|           |
//			|   Cyan    |
//			|           |
//			|___________|
//			|           |
//			|           |
//			|   Black   |
//			|           |
//			|___________|
//			|           |
//			|           |
//			|   Gray	|
//			|           |
//			|___________|
///////////////////////////////////////////////////////


BOOL ShareRVBtoMem(FILE *file,unsigned char *lpMem, long Height, long Width, unsigned int WidthLine, long OffBits)
{
 unsigned char *lpImgBuf, *lpYellow, *lpMagenta, *lpCyan, *lpBlack,*lpGray;
 unsigned char	red,green,blue,r,g,b;
 long	index_line, index_dot;
 BOOL	stpt = FALSE;

 lpImgBuf = new BYTE[WidthLine];
 if(!lpImgBuf) return(FALSE);
	
 lpYellow  = &lpMem[0]; 
 lpMagenta = &lpMem[Width * Height];  
 lpCyan    = &lpMem[2 * Width * Height]; 
 lpBlack   = &lpMem[3 * Width * Height]; 
 lpGray   =  &lpMem[4 * Width * Height]; 

 ///////////////////
 // PORTRAIT CASE //
 ///////////////////
 if(Height >= Width)
 {
	 for(index_line = 0; index_line < Height; index_line++)
	 {
		fseek(file,(OffBits + (WidthLine * index_line)),SEEK_SET);
		if(fread(lpImgBuf,sizeof(BYTE),WidthLine,file) == WidthLine)
		{
			r = 0xFF;
			g = 0xFF;
			b = 0xFF;
			stpt = FALSE;
			for(index_dot = Width ; index_dot > 0; index_dot--)
			{
				blue	= lpImgBuf[(index_dot*3)];
				green	= lpImgBuf[(index_dot*3)+1];
				red		= lpImgBuf[(index_dot*3)+2];
				*(lpGray++)	= (unsigned char) RgbToGray(red,green,blue);
				if( blue == 0x00 && green == 0x00 && red == 0x00)
				{
					if(!stpt)
					{
						*(lpYellow++)	= 0x00;
						*(lpMagenta++)	= 0x00;
						*(lpCyan++)		= 0x00;
						*(lpBlack++)	= 0xFF;
						stpt = TRUE;
					}
					else
					{	
						*(lpYellow++)	= ((0xFF - b));
						*(lpMagenta++)	= ((0xFF - g));
						*(lpCyan++)		= ((0xFF - r));
						*(lpBlack++)	= (0xFF);
					}
				}
				else
				{
					r = red;
					g = green;
					b = blue;
					*(lpYellow++)	= ((0xFF - b));
					*(lpMagenta++)	= ((0xFF - g));
					*(lpCyan++)		= ((0xFF - r));
					*(lpBlack++)	= 0x00;
					stpt = TRUE;
				}
			 }
		}
	 }
 }
 ////////////////////
 // LANDSCAPE CASE //
 ///////////////////
 else 
 {
	 for(index_line = 0; index_line < Height ; index_line++)
	 {
		r = 0xFF;
		g = 0xFF;
		b = 0xFF;
		stpt = FALSE;
		fseek(file,OffBits + (index_line*WidthLine),SEEK_SET);
		if(fread(lpImgBuf,sizeof(BYTE),WidthLine,file) == WidthLine)
		{
			for(index_dot = 0 ; index_dot < Width ; index_dot++)
			{
				blue	= lpImgBuf[(index_dot *3)];
				green	= lpImgBuf[(index_dot *3) + 1];
				red		= lpImgBuf[(index_dot *3) + 2];
				lpGray[(index_line) + (index_dot*Height)] =  (unsigned char)RgbToGray(red,green,blue);
				if( blue == 0x00 && green == 0x00 && red == 0x00)
				{
					if(!stpt)
					{
						lpYellow[(index_line) + (index_dot*Height)]	= 0x00;
						lpMagenta[(index_line)+ (index_dot*Height)]	= 0x00;
						lpCyan[(index_line) + (index_dot*Height)]	= 0x00;
						lpBlack[(index_line) + (index_dot*Height)]	= 0xFF;
						stpt = TRUE;
					}
					else
					{	
						lpYellow[(index_line) + (index_dot*Height)]	= ((0xFF - b));
						lpMagenta[(index_line)+ (index_dot*Height)]	= ((0xFF - g));
						lpCyan[(index_line) + (index_dot*Height)]	= ((0xFF - r));
						lpBlack[(index_line) + (index_dot*Height)]	= (0xFF);
					}
				}
				else
				{
					r = red;
					g = green;
					b = blue;
					lpYellow[(index_line) + (index_dot*Height)]	= ((0xFF - b));
					lpMagenta[(index_line)+ (index_dot*Height)]	= ((0xFF - g));
					lpCyan[(index_line) + (index_dot*Height)]	= ((0xFF - r));
					lpBlack[(index_line) + (index_dot*Height)]	= 0x00;
					stpt = TRUE;
				}
			 }
		}
	 }
 }
 delete(lpImgBuf);
 return(TRUE);
}


//===============================================================================//
// Reduction of the color data according to the number of bits per dot choosen 
// Input  : 1 panel (1016 * 648)
// Output : 1 panel of usefull bit ((1016*648)*uiBitComp)/8
//===============================================================================//
long ReduceColor(unsigned char *lpMemIn ,unsigned char *lpMemOut, unsigned int uiBitComp,int bPartial,long Max)
{
 long			lIndex = 0, lIndex1=0 ;
 unsigned char 	*lpbBuf, *lpbData, *lpbDataOut;
 long			lComp = 0;	
 long 			lNbrByte;

 if(!bPartial)  lNbrByte=(1016*648);
 else lNbrByte=((Max)*648);

 lpbBuf = new unsigned char[8];
 if(lpbBuf == 0) {
        return(0);
  }
 lpbData = &lpMemIn[0];
 lpbDataOut = &lpMemOut[0];
 while(lIndex < lNbrByte) {
	 for( lIndex1 = 0; lIndex1 < 8; lIndex1++) {
		if( lIndex < lNbrByte) {
		  lpbBuf[lIndex1] = *(lpbData++) >> (8-uiBitComp) ;
		  lIndex ++;
		}
		else {
			lpbBuf[lIndex1] = 0x00;
		}
	 }
	 switch(uiBitComp) {
	 case 6 :
		*(lpbDataOut++) = ( lpbBuf[0]<<2 ) | ( lpbBuf[1]>>4 );		
		*(lpbDataOut++) = ( lpbBuf[1]<<4 ) | ( lpbBuf[2]>>2 );
		*(lpbDataOut++) = ( lpbBuf[2]<<6 ) | ( lpbBuf[3]);
		*(lpbDataOut++) = ( lpbBuf[4]<<2 ) | ( lpbBuf[5]>>4 );		
		*(lpbDataOut++) = ( lpbBuf[5]<<4 ) | ( lpbBuf[6]>>2 );
		*(lpbDataOut++) = ( lpbBuf[6]<<6 ) | ( lpbBuf[7]);
		break;
	 case 7:
		*(lpbDataOut++) = ( lpbBuf[0]<<1 ) | ( lpbBuf[1]>>6 );		
		*(lpbDataOut++) = ( lpbBuf[1]<<2 ) | ( lpbBuf[2]>>5 );
		*(lpbDataOut++) = ( lpbBuf[2]<<3 ) | ( lpbBuf[3]>>4 );
		*(lpbDataOut++) = ( lpbBuf[3]<<4 ) | ( lpbBuf[4]>>3 );		
		*(lpbDataOut++) = ( lpbBuf[4]<<5 ) | ( lpbBuf[5]>>2 );
		*(lpbDataOut++) = ( lpbBuf[5]<<6 ) | ( lpbBuf[6]>>1 );
		*(lpbDataOut++) = ( lpbBuf[6]<<7 ) | ( lpbBuf[7] );
		break;
	 default : //5 bits
		*(lpbDataOut++) = ( lpbBuf[0]<<3 ) | ( lpbBuf[1]>>2 );		
		*(lpbDataOut++) = ( lpbBuf[1]<<6 ) | ( lpbBuf[2]<<1 ) | ( lpbBuf[3]>>4);
		*(lpbDataOut++) = ( lpbBuf[3]<<4 ) | ( lpbBuf[4]>>1 );
		*(lpbDataOut++) = ( lpbBuf[4]<<7 ) | ( lpbBuf[5]<<2 ) | ( lpbBuf[6]>>3);
		*(lpbDataOut++) = ( lpbBuf[6]<<5 ) | ( lpbBuf[7] );
		//break;
	 }
 }
 delete(lpbBuf);
 lComp = ( ((lNbrByte*10)/8) * uiBitComp);
 lComp /= 10;
 if(lNbrByte%8) lComp++;
 return(lComp);
}

//===================================================//
// Black panel and overlay ONLY 
// input  : 1 byte = 1 pixel
// output : 1 byte = 8 pixels
//===================================================//
long ReduceBlack(unsigned char *lpMemIn ,unsigned char *lpMemOut)
{
 long			lIndex = 0, lIndex1=0 ;
 unsigned char 	*lpbData, *lpbDataOut;
 long			lComp = 0;	
 unsigned char	bBuf = 0x00;
 unsigned char	Mask[8] = {0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x01};
 long 			lNbrByte = 1016 * 648;
 lpbData = &lpMemIn[0];
 lpbDataOut = &lpMemOut[0];
 while(lIndex < lNbrByte) {
	 for( lIndex1 = 0; lIndex1 < 8; lIndex1++) {
		if(lIndex < lNbrByte) {
			if(*(lpbData++) ) {
				bBuf |= Mask[lIndex1];
			}
			lIndex++;
		}
	 }
	*(lpbDataOut++) = bBuf;
	bBuf = 0x00;
	lComp++;
 }
 return(lComp);
}


//===============================================================================//
// Reduction of the color data according to the number of bits per dot choosen 
// Input  : 1 panel (1016 * 648)
// Output : compressed data
//===============================================================================//
long CompressColor5(unsigned char *lpMemIn ,unsigned char *lpMemOut,int bPartial,long Max)//, unsigned int uiBitComp)
{
 long			lIndex = 0, lIndex1=0 , i = 0;
 unsigned char 	cur,prev, *lpbData, *lpbDataOut;
 long			lComp = 0;	
 long 			lNbrByte;
 
 if(bPartial == 0) lNbrByte=(1016*648);
 else lNbrByte=((Max)*648);

 lpbData = &lpMemIn[0];
 lpbDataOut = &lpMemOut[0];
 cur = lpbData[lIndex] >> (8-5);
 prev = cur;
 while(lIndex < lNbrByte) {
	  cur = lpbData[lIndex] >> (8-5);
	  lIndex1 = 1;
	  while(cur == prev && lIndex < lNbrByte) {
		  cur = lpbData[lIndex] >> (8-5);
		  lIndex1++;
		  lIndex++;

	  }
	  if(prev == 0x00) {
		  while(lIndex1 > 648) {//blank line
			  lpbDataOut[lComp] = 0x80;
			  lComp++;
			  lpbDataOut[lComp] = 0x00;
			  lComp++;				
			  lIndex1 -= 648;
		  }
		  if(lIndex1 == 1) { // isolated dot
			  lpbDataOut[lComp] = 0x00;
			  lComp++;
		  }
		  else { 
			  i = lIndex1 / 256;
			  if(!i) {
				  lpbDataOut[lComp] = 0x80;
				  lComp++;
				  lpbDataOut[lComp] = (unsigned char)lIndex1;
				  lComp++;
			  }
			  else {
				  if(i==1) {
					  lpbDataOut[lComp] = 0xA0;
					  lComp++;
					  lpbDataOut[lComp] = (unsigned char)(lIndex1 - 256);
					  lComp++;
				  }
				  else {
					  if(i==2) {
						  lpbDataOut[lComp] = 0xE0;
						  lComp++;
						  lpbDataOut[lComp] = (unsigned char)(lIndex1 - 512);
						  lComp++;
					  }
				  }
			  }
		  }
	  }
	  else { 
		  while(lIndex1 > 767) {
			  lpbDataOut[lComp] = (0x1F & prev) | 0xE0;
			  lComp++;
			  lpbDataOut[lComp] = 0xFF;
			  lComp++;
			  lIndex1 -= 767;
		  }
		  if(lIndex1 == 1) { // isolated dot
			  lpbDataOut[lComp] = 0x1F & prev;
			  lComp++;
		  }
		  else { 
			  i = lIndex1 / 256;
			  if(!i) {
				  lpbDataOut[lComp] = (0x1F & prev) | 0x80;
				  lComp++;
				  lpbDataOut[lComp] = (unsigned char)lIndex1;
				  lComp++;
			  }
			  else {
				  if(i==1) {
					  lpbDataOut[lComp] = (0x1F & prev) | 0xA0;
					  lComp++;
					  lpbDataOut[lComp] = (unsigned char)(lIndex1 - 256);
					  lComp++;
				  }
				  else {
					  if(i==2) {
						  lpbDataOut[lComp] = (0x1F & prev) | 0xE0;
						  lComp++;
						  lpbDataOut[lComp] = (unsigned char)(lIndex1 - 512);
						  lComp++;
					  }
				  }
			  }
		  } 
	  }
	  prev = cur;
	  lIndex++;
 }
 return(lComp);
}

//===============================================================================//
// Reduction of the color data according to the number of bits per dot choosen 
// Input  : 1 panel (1016 * 648)
// Output : compressed data
//===============================================================================//
long CompressColor6(unsigned char *lpMemIn ,unsigned char *lpMemOut,int bPartial,long Max)//, unsigned int uiBitComp)
{
 long			lIndex = 0, lIndex1=0 , i = 0;
 unsigned char 	cur,prev, *lpbData, *lpbDataOut;
 long			lComp = 0;	
 long 			lNbrByte;
 
 if(bPartial == 0) lNbrByte=(1016*648);
 else lNbrByte=((Max)*648);

 lpbData = &lpMemIn[0];
 lpbDataOut = &lpMemOut[0];
 cur = lpbData[lIndex] >> (8-6);
 prev = cur;
 while(lIndex < lNbrByte) {
	  cur = lpbData[lIndex] >> (8-6);
	  lIndex1 = 1;
	  while(cur == prev && lIndex < lNbrByte) {
		  cur = lpbData[lIndex] >> (8-6);
		  lIndex1++;
		  lIndex++;

	  }
	  if(prev == 0x00) {
		  while(lIndex1 > 648) {//blank line
			  lpbDataOut[lComp] = 0x80;
			  lComp++;
			  lpbDataOut[lComp] = 0x00;
			  lComp++;				
			  lIndex1 -= 648;
		  }
		  if(lIndex1 == 1) { // isolated dot
			  lpbDataOut[lComp] = 0x00;
			  lComp++;
		  }
		  else {
			  while(lIndex1 > 511) {
				  lpbDataOut[lComp] = (0x3F & prev) | 0xC0;
				  lComp++;
				  lpbDataOut[lComp] = 0xFF;
				  lIndex1 -= 511;
				  lComp++;
			  }
			  if(lIndex1 > 256) {
				  lpbDataOut[lComp] = (0x3F & prev) | 0xC0;
				  lComp++;
				  lpbDataOut[lComp] = (unsigned char)(lIndex1 - 256);
				  lComp++;
			  }
			  else {
				  lpbDataOut[lComp] = (0x3F & prev) | 0x80;
				  lComp++;
				  lpbDataOut[lComp] = (unsigned char)(lIndex1);
				  lComp++;
			  }
		  }
	  }
	  else { 
		  while(lIndex1 > 511) {
				  lpbDataOut[lComp] = (0x3F & prev) | 0xC0;
				  lComp++;
				  lpbDataOut[lComp] = 0xFF;
				  lIndex1 -= 511;
				  lComp++;
		  }
		  if(lIndex1 == 1) { // isolated dot
			  lpbDataOut[lComp] = 0x3F & prev;
			  lComp++;
		  }
		  else { 
			  if(lIndex1 > 256) {
				  lpbDataOut[lComp] = (0x3F & prev) | 0xC0;
				  lComp++;
				  lpbDataOut[lComp] = (unsigned char)(lIndex1 - 256);
				  lComp++;
			  }
			  else {
				  lpbDataOut[lComp] = (0x3F & prev) | 0x80;
				  lComp++;
				  lpbDataOut[lComp] = (unsigned char)(lIndex1 );
				  lComp++;
			  }
		  }
	  }
	  prev = cur;
	  lIndex++;
 }
 return(lComp);
}


//===============================================================================//
// Reduction of the color data according to the number of bits per dot choosen 
// Input  : 1 panel (1016 * 648)
// Output : compressed data
//===============================================================================//
long CompressColor7(unsigned char *lpMemIn ,unsigned char *lpMemOut,int bPartial,long Max)
{
 long			lIndex = 0, lIndex1=0 , i = 0;
 unsigned char 	cur,prev, *lpbData, *lpbDataOut;
 long			lComp = 0;	
 long 			lNbrByte;
 
 if(bPartial == 0) lNbrByte=(1016*648);
 else lNbrByte=((Max)*648);

 lpbData = &lpMemIn[0];
 lpbDataOut = &lpMemOut[0];
 cur = lpbData[lIndex] >> (8-7);
 prev = cur;

 while(lIndex < lNbrByte) {
	  cur = lpbData[lIndex] >> (8-7);
	  lIndex1 = 1;
	  while(cur == prev && lIndex < lNbrByte) {
		  cur = lpbData[lIndex] >> (8-7);
		  lIndex1++;
		  lIndex++;

	  }
	  if(prev == 0x00) {
		  while(lIndex1 > 648) {//blank line
			  lpbDataOut[lComp] = 0x80;
			  lComp++;
			  lpbDataOut[lComp] = 0x00;
			  lComp++;				
			  lIndex1 -= 648;
		  }
		  if(lIndex1 == 1) { //  isolated dot
			  lpbDataOut[lComp] = 0x00;
			  lComp++;
		  }
		  else { 
			  while(lIndex1 > 255) {
				  lpbDataOut[lComp] = (0x7F & prev) | 0x80;
				  lComp++;
				  lpbDataOut[lComp] = 0xFF;
				  lIndex1 -= 255;
				  lComp++;
			  }
			  lpbDataOut[lComp] = (0x7F & prev) | 0x80;
			  lComp++;
			  lpbDataOut[lComp] = (unsigned char)(lIndex1);
			  lComp++;
		  }
	  }
	  else { 
		  while(lIndex1 > 255) {
			  lpbDataOut[lComp] = (0x7F & prev) | 0x80;
			  lComp++;
			  lpbDataOut[lComp] = 0xFF;
			  lComp++;
			  lIndex1 -= 255;
		  }
		  if(lIndex1 == 1) {  // isolated dot
			  lpbDataOut[lComp] = 0x7F & prev;
			  lComp++;
		  }
		  else { 
			  lpbDataOut[lComp] = (0x7F & prev) | 0x80;
			  lComp++;
			  lpbDataOut[lComp] = (unsigned char)(lIndex1);
			  lComp++;
		  }
	  }
	  prev = cur;
	  lIndex++;
 }
 return(lComp);
}

//===============================================================================//
// Reduction of the monochrome data 
// Input  : 1 panel (1016 * 648)
// Output : compressed data
//===============================================================================//
long CompressBlack(unsigned char *lpMemIn ,unsigned char *lpMemOut)
{
 long			lIndex = 0, lIndex1=0 , i = 0;
 unsigned char 	*lpbData, *lpbDataOut;
 long			lComp = 0;	
 long 			lNbrByte = (1016*648)/8;
 unsigned char  lpTemp[82];
 int			line = 0,index = 0;
 
 lpbData = &lpMemIn[0];
 lpbDataOut = &lpMemOut[0];
 
 while(lIndex < lNbrByte) {
	  lIndex1 = 0;
	  line = 0;
	  index = 1;
	  lpTemp[lIndex1] = lpbData[lIndex] ;
	  lIndex1++;
	  lIndex++;

	  while(lIndex1 < 81) {
		  lpTemp[lIndex1] = lpbData[lIndex] ;
		  if((lpTemp[lIndex1-1] != lpTemp[lIndex1])) line = 1;
		  if(lpTemp[lIndex1-1]) index = lIndex1;
		  lIndex1++;
		  lIndex++;
	  }
	  if(!lpTemp[lIndex1-1] && !line) {//white line
		  lpbDataOut[lComp] = 0x00;
		  lComp++;
	  }
	  else {
		   if(lpTemp[lIndex1-1] && !line) {//black line
				lpbDataOut[lComp] = 0xFF;
				lComp++;
		   }
		   else {
			   	lpbDataOut[lComp] = index;
				lComp++;
				for(i=0;i<(index);i++) {
					lpbDataOut[lComp] = lpTemp[i];
					lComp++;
				}
		   }
	  }
 }
 return(lComp);
}



//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
BOOL ShareKCMYtoMem(FILE *file,unsigned char *lpMem, long Height, long Width, unsigned int WidthLine)
{
 unsigned char *lpImgBuf, *lpYellow, *lpMagenta, *lpCyan, *lpBlack;//,*lpGray;
 long	index_line, index_dot,i;

 lpImgBuf = new BYTE[WidthLine];
 if(!lpImgBuf) return(FALSE);
	
 lpYellow  = &lpMem[0]; 
 lpMagenta = &lpMem[Width * Height];  
 lpCyan    = &lpMem[2 * Width * Height]; 
 lpBlack   = &lpMem[3 * Width * Height]; 

 //////////////////
 // PORTRAIT CASE//
 //////////////////
 
 if(Height >= Width)
 {
 for(index_line = 0; index_line < Height; index_line++)
 {
	fseek(file,((WidthLine * index_line)),SEEK_SET);
	if(fread(lpImgBuf,sizeof(BYTE),WidthLine,file) == WidthLine)
	{
		for(index_dot = Width ; index_dot > 0; index_dot--)
		{
			*(lpBlack++)	= lpImgBuf[(index_dot*4)];
			*(lpCyan++)		= lpImgBuf[(index_dot*4)+1];
			*(lpMagenta++)	= lpImgBuf[(index_dot*4)+2];
			*(lpYellow++)	= lpImgBuf[(index_dot*4)+3];
		 }
	}
 }
 }
 //////////////////
 //LANDSCAPE CASE//
 //////////////////
 else 
 {
 for(index_line = 0; index_line < Height ; index_line++)
 {
	fseek(file,(index_line*WidthLine),SEEK_SET);
	if(fread(lpImgBuf,sizeof(BYTE),WidthLine,file) == WidthLine)
	{
		i=0;
		for(index_dot = Width-1 ; index_dot > 0; index_dot--)
		{
			lpYellow[(index_line) + (i*Height)]	= lpImgBuf[((index_dot) *4) + 3];
			lpMagenta[(index_line)+ (i*Height)]	= lpImgBuf[((index_dot) *4) + 2];
			lpCyan[(index_line) + (i*Height)]	= lpImgBuf[((index_dot) *4) + 1];
			lpBlack[(index_line) + (i*Height)]	= lpImgBuf[((index_dot) *4)];
			i++;
		 }
	}
 }
 }
 delete(lpImgBuf);
 return(TRUE);
}

