/*
    This file is part of the Evolis SDK.

    The Evolis SDK is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The Evolis SDK is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with the Evolis SDK.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "stdafx.h"
#include "Bitmap.h"
#include "evolib.h"


//===============================================================//
// Single 5 bits uncompress panel Downloading 
// Input : ucData / raw data
// 		color /y,m or c	
// Return: 	0 if OK
// 		-1 if writing error   
//===============================================================//

int DB32NC(unsigned char * lpMem, long lPos, char color, unsigned char *ucCommand, long lCommandSize, int bPartial, long Min, long Max)
{
	 unsigned char *ucCol;
	 long 		lComp =0;
 
	 if(!ucCommand) {
		return(0);
	 }

	 if (bPartial == 0) {
		ucCol = &lpMem[lPos];
		strcpy((char *)ucCommand, "\033Db;y;32;");
		strncpy((char *)&ucCommand[4],&color,1);    // replace 'y' by color
		lCommandSize = ReduceColor(ucCol ,&ucCommand[9], 5,bPartial,Max);
		strncpy((char *)&ucCommand[lCommandSize+9],"\015",1);
		lCommandSize += 10;
	 }
	 else {
		ucCol = &lpMem[lPos+(Min*648)];
		strcpy((char *)ucCommand, "\033Dbp;y;32;");
		strncpy((char *)&ucCommand[5],&color,1);
		sprintf((char *)&ucCommand[10], "%.4ld;%.3ld;",Min,Max);
		lCommandSize = ReduceColor(ucCol ,&ucCommand[19], 5,bPartial,Max);
		strncpy((char *)&ucCommand[lCommandSize+19],"\015",1);
		lCommandSize += 20;
	 }
	 return(lCommandSize);
}

//===============================================================//
// Single 6 bits uncompress panel Downloading 
// Input : ucData / raw data
// 		color /y,m or c	
// Return: 	0 if OK
// 		-1 if writing error   
//===============================================================//
int DB64NC(unsigned char * lpMem, long lPos, char color, unsigned char *ucCommand, long lCommandSize, int bPartial, long Min, long Max)
{
	 unsigned char *ucCol;

	 if(!ucCommand) {
		return(0);
	 }
 
	 if(bPartial == 0) {
		ucCol = &lpMem[lPos];
		strcpy((char *)ucCommand, "\033Db;y;64;");
		strncpy((char *)&ucCommand[4],&color,1);
		lCommandSize = ReduceColor(ucCol ,&ucCommand[9], 6,bPartial,Max);
		strncpy((char *)&ucCommand[lCommandSize+9],"\015",1);
		lCommandSize += 10;
	 }
	 else
	 {
		ucCol = &lpMem[lPos+(Min*648)];
		strcpy((char *)ucCommand, "\033Dbp;y;64;");
		strncpy((char *)&ucCommand[5],&color,1);
		sprintf((char *)&ucCommand[10], "%.4ld;%.3ld;",Min,Max);
		lCommandSize = ReduceColor(ucCol ,&ucCommand[19], 6,bPartial,Max);
		strncpy((char *)&ucCommand[lCommandSize+19],"\015",1);
		lCommandSize += 20;
	 }
	 return(lCommandSize);
}

//===============================================================//
// Single 7 bits uncompress panel Downloading 
// Input : ucData / raw data
// 		color /y,m or c	
// Return: 	0 if OK
// 		-1 if writing error   
//===============================================================//
int DB128NC(unsigned char * lpMem, long lPos, char color, unsigned char *ucCommand, long lCommandSize, int bPartial, long Min, long Max)
{
	 unsigned char *ucCol;

	 if(!ucCommand) {
		return(0);
	 }

	 if(bPartial == 0) {
		ucCol = &lpMem[lPos];
		strcpy((char *)ucCommand, "\033Db;y;128;");
		strncpy((char *)&ucCommand[4], &color , 1 );
		lCommandSize = ReduceColor(ucCol ,&ucCommand[10], 7,bPartial,Max);
		strncpy((char *)&ucCommand[lCommandSize+10], "\015" , 1 );
		lCommandSize += 11;
	 }
	 else {
		ucCol = &lpMem[lPos+(Min*648)];
		strcpy((char *)ucCommand, "\033Dbp;y;128;");
		strncpy((char *)&ucCommand[5],&color,1);
		sprintf((char *)&ucCommand[11], "%.4ld;%.3ld;",Min,Max);
		lCommandSize = ReduceColor(ucCol ,&ucCommand[20], 7,bPartial,Max);
		strncpy((char *)&ucCommand[lCommandSize+20],"\015",1);
		lCommandSize += 21;
	 }
	 return(lCommandSize);
}

//===============================================================//
// Black and overlay uncompress panels Downloading 
// Input : ucData / raw data
// 		color /k, full varnish	
// Return: 	0 if OK
// 		-1 if writing error   
//===============================================================//
int DB2NC(unsigned char * lpMem, long lPos, char pannel[10], unsigned char *ucCommand, long lCommandSize)
//(long lPos, char pannel[10])
{
	unsigned char *ucCol;
	int numwritten = 0;
	int B, T, L, R, i = 0, j = 0;
	int pB, pT, ModuloB, ModuloT;

	unsigned char MaskB[8] = { 0x00, 0x80, 0xC0, 0xE0, 0xF0, 0xF8, 0xFC, 0xFE };
	unsigned char MaskT[8] = { 0x00, 0x01, 0x03, 0x07, 0x0F, 0x1F, 0x3F, 0x7F };

	if (!ucCommand) {
		return (0);
	}

	 lCommandSize = 0;
	if (!(strcmp(pannel, "ABP"))) {
		ucCol = &lpMem[lPos];
		strcpy((char *)ucCommand, "\033Db;k;2;");
		lCommandSize = ReduceBlack(ucCol, &ucCommand[8]);
		strncpy((char *)&ucCommand[lCommandSize + 8], "\015", 1);
		lCommandSize += 9;
	}
	else if (!(strcmp(pannel, "NO"))) {		//No Varnish
			memset(ucCommand, 0x00, lCommandSize);
			strcpy((char *)ucCommand, "\033Db;o;2;");
			strncpy((char *)&ucCommand[lCommandSize + 8], "\015", 1);
			lCommandSize += 9;
	}
	else if (!(strcmp(pannel, "FO"))) {	//Full Varnish
			memset(ucCommand, 0xFF, lCommandSize);
			strcpy((char *)ucCommand, "\033Db;o;2;");
			strncpy((char *)&ucCommand[lCommandSize + 8], "\015", 1);
			lCommandSize += 9;
	}
	else {
	  if (!(strcmp(pannel, "SCI"))) {
		  B = 319;
		  T = 106;
		  L = 71;
		  R = 307;
		  memset(ucCommand, 0xFF, lCommandSize);
	  }
	  else if (!(strcmp(pannel, "SCA"))) {
		  B = 354;
		  T = 177;
		  L = 71;
		  R = 283;
		  memset(ucCommand, 0xFF, lCommandSize);
	  } 
	  else if (!(strcmp(pannel, "MS"))) {
		  B = 216;
		  T = 60;
		  L = 0;
		  R = 1016;
		  memset(ucCommand, 0xFF, lCommandSize);
	  }
	  else {
		  return (0);
	  }
	  //White part
	  for (j = L; j <= R; j++) {
		  pB = (j * 648) + (648 - B);
		  ModuloB = pB % 8;
		  pB /= 8;
		  if (ModuloB)
			  pB++;
		  ucCommand[pB + 7] = MaskB[ModuloB];
		  pB++;
		  pT = (j * 648) + (648 - T);
		  ModuloT = pT % 8;
		  pT /= 8;
		  if (ModuloT)
			  pT++;
		  ucCommand[pT + 7] = MaskT[ModuloT];
		  pT--;
		  for (i = (pB); i <= (pT); i++) {
			  ucCommand[i + 7] = 0x00;
		  }
	  }//end of white part
	  strcpy((char *)ucCommand, "\033Db;o;2;");
	  strncpy((char *)&ucCommand[lCommandSize + 8], "\015", 1);
	  lCommandSize += 9;
	}
	return (lCommandSize);
}


//===============================================================//
// Single 5 bits compressed panel Downloading 
// Input : ucData / raw data
// 		color /y,m or c	
// Return: 	0 if OK
// 		-1 if writing error   
//===============================================================//
int DB32C(unsigned char * lpMem, 
		  long lPos, 
		  char color, 
		  unsigned char *ucCommand, 
		  long lCommandSize, 
		  int bPartial, 
		  long Min, 
		  long Max)
{
	 unsigned char 		*ucCol;
	 long 				lComp =0;
	 int 				numwritten =0;
	 char				entete[18];

	  if(!ucCommand) {
		return(0);
	  }
	 if(bPartial == 0) {
		lCommandSize = 18;
		ucCol = &lpMem[lPos];
		lComp = CompressColor5(ucCol ,&ucCommand[17],bPartial,Max);
		lCommandSize += lComp;
		sprintf(entete,"\033Dbc;%c;32;%06u;",color,lComp);
		strncpy((char *)&ucCommand[0],entete,17);
		strncpy((char *)&ucCommand[lCommandSize++],"\015",1);
		return(lCommandSize);
	 }
	 else {
		lCommandSize = 24;
		ucCol = &lpMem[lPos+(Min*648)];
		lComp = CompressColor5(ucCol ,&ucCommand[23],bPartial,Max);
		lCommandSize += lComp;
		sprintf(entete,"\033Dbpc;%c;32;%.4ld;%06u;",color,Min,lComp);
		strncpy((char *)&ucCommand[0],entete,23);
		strncpy((char *)&ucCommand[lCommandSize++],"\015",1);
		return(lCommandSize);
	 }
}

//===============================================================//
// Single 6 bits compressed panel Downloading 
// Input : ucData / raw data
// 		color /y,m or c	
// Return: 	0 if OK
// 		-1 if writing error   
//===============================================================//
int DB64C(unsigned char * lpMem, long lPos, char color, unsigned char *ucCommand, long lCommandSize, int bPartial, long Min, long Max)
{
	 unsigned char 		*ucCol;
	 long 				lComp =0;
	 char				entete[18];

	 if(!ucCommand) {
		return(0);
	 }
	 if(bPartial == 0) {
		lCommandSize = 18;
		ucCol = &lpMem[lPos];
		lComp = CompressColor6(ucCol ,&ucCommand[17],bPartial,Max);
		lCommandSize += lComp;
		sprintf(entete,"\033Dbc;%c;64;%06u;",color,lComp);
		strncpy((char *)&ucCommand[0],entete,17);
		strncpy((char *)&ucCommand[lCommandSize++],"\015",1);
		return(lCommandSize);
	 }
	 else {
		lCommandSize = 24;
		ucCol = &lpMem[lPos+(Min*648)];
		lComp = CompressColor6(ucCol ,&ucCommand[23],bPartial,Max);
		lCommandSize += lComp;
		sprintf(entete,"\033Dbpc;%c;64;%.4ld;%06u;",color,Min,lComp);
		strncpy((char *)&ucCommand[0],entete,23);
		strncpy((char *)&ucCommand[lCommandSize++],"\015",1);
		return(lCommandSize);
	 }

 }
//===============================================================//
// Single 7 bits compressed panel Downloading 
// Input : ucData / raw data
// 		color /y,m or c	
// Return: 	0 if OK
// 		-1 if writing error   
//===============================================================//
int DB128C(unsigned char * lpMem, long lPos, char color, unsigned char *ucCommand, long lCommandSize, int bPartial, long Min, long Max)
{
	 unsigned char 		*ucCol;
	 long 				lComp =0;
	 char				entete[256];

	  if(!ucCommand) {
		return(0);
	 }
 
	 if(bPartial == 0)
	 {
		lCommandSize = 19;
		ucCol = &lpMem[lPos];
		lComp = CompressColor7(ucCol ,&ucCommand[18],bPartial,Max);
		lCommandSize += lComp;
		sprintf(entete,"\033Dbc;%c;128;%06u;",color,lComp);
		strncpy((char *)&ucCommand[0],entete,18);
		strncpy((char *)&ucCommand[lCommandSize++],"\015",1);
		return(lCommandSize);
	 }
	 else
	 {
		lCommandSize = 25;
		ucCol = &lpMem[lPos+(Min*648)];
		lComp = CompressColor7(ucCol ,&ucCommand[24],bPartial,Max);
		lCommandSize += lComp;
		sprintf(entete,"\033Dbpc;%c;128;%.4ld;%06u;",color,Min,lComp);
		strncpy((char *)&ucCommand[0],entete,24);
		strncpy((char *)&ucCommand[lCommandSize++],"\015",1);
		return(lCommandSize);
	 }
}

 //===============================================================//
// black and overlay compressed panels Downloading 
// Input : ucData / raw data
// 		color /k overlay	
// Return: 	0 if OK
// 		-1 if writing error   
//===============================================================//

int DB2C(unsigned char * lpMem, long lPos, char pannel[10], unsigned char *ucCommand, long lCommandSize)
{
	 unsigned char *ucCol,*ucTemp;
	 long 		lComp;
	 int 		numwritten = 0; 
	 char		entete[18];
	 
	 if(!ucCommand) {
		return(0);
	 }

	 ucTemp = new unsigned char[lCommandSize];
	 if(!ucTemp) {
		return(0);
	 }
	 lCommandSize = 0;
 
	 if (!(strcmp(pannel, "ABP"))) {
		ucCol = &lpMem[lPos];
		lCommandSize = 17;
		lComp = ReduceBlack(ucCol ,ucTemp);
		lComp = CompressBlack(ucTemp,&ucCommand[16]);
		sprintf(entete,"\033Dbc;k;2;%06u;",lComp);
		lCommandSize += lComp;
		strncpy((char *)&ucCommand[0],entete,16);
		strncpy((char *)&ucCommand[lCommandSize++],"\015",1);    
	 }
	 else if (!(strcmp(pannel, "FO"))) {//Full Varnish

			memset(ucTemp,0xFF,lCommandSize);
			lCommandSize = 17;
			lComp = CompressBlack(ucTemp,&ucCommand[16]);
			sprintf(entete,"\033Dbc;o;2;%06u;",lComp);
			lCommandSize += lComp;
			strncpy((char *)&ucCommand[0],entete,16);
			strncpy((char *)&ucCommand[lCommandSize++],"\015",1);
	 }
	 delete(ucTemp);
	 return(lCommandSize);
 }