/*
    This file is part of the Evolis SDK.

    The Evolis SDK is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The Evolis SDK is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with the Evolis SDK.  If not, see <http://www.gnu.org/licenses/>.
*/

/////////////////RVB TO GRAY TO BLACK

#include "stdafx.h"
extern	long ReduceBlack(unsigned char *,unsigned char * );

#include "evolib.h"

#define	RgbToGray(r, g, b) (((long) (r)*74L + (long)(g)*155L +(long)(b)*27L) >> 8)

int		DitherPattern1[8][8] = {
	{  0,112, 28,140 , 7,119, 35,147 },
	{168, 56,196, 84,175, 63,203, 78 },
	{ 42,154, 14,126, 49,161, 21,114 },
	{210, 98,182, 70,217,105,189, 77 },
	{ 11,123, 39,151,  4,116, 32,143 },
	{179, 67,207, 95,172, 59,200, 88 },
	{ 53,165, 24,137, 46,158, 18,111 },
	{221,109,193, 81,214,102,159, 74 }
};


//////////////////////////////////////////////////////////////////////////////////////
///////								RVB TO GRAY  
/////// Input parameters
///////				lpMemIn	: RVB data (3 byte by dot)
///////				lNbrByte: number of byte to deal
/////// Output parameter
///////				lpMemOut: pointer on the gray data, result of the conversion
//////////////////////////////////////////////////////////////////////////////////////
void RVBtoGray(unsigned char *lpMemIn , unsigned char *lpMemOut, long lNbrByte)
{
 long	lIndex = 0;
 unsigned char *lpbData,*lpbDataOut;
 unsigned char red,green,blue;
 long	d;
 lpbData = &lpMemIn[0];
 lpbDataOut = &lpMemOut[0];
 while(lIndex < lNbrByte) {
  		blue	= lpbData[(lIndex*3)];
		green	= lpbData[(lIndex*3)+1];
		red		= lpbData[(lIndex*3)+2];
		d		= RgbToGray(red, green, blue);
		lpbDataOut[lIndex]		= (BYTE)d;
		lIndex++;
 }
}
//////////////////////////////////////////////////////////////////////////////////////
///////								GRAY  THRESHOLD
/////// Input parameters
///////				lpMemIn	: gray data (1 byte by dot)
///////				lNbrByte: number of byte to deal
/////// Output parameter
///////				lpMemOut: pointer on the monochrome data, result of the gray conversion
//////////////////////////////////////////////////////////////////////////////////////
void GrayToThreshold(unsigned char *lpMemIn ,unsigned char *lpMemOut, long lNbrByte)
{
 long	lIndex = 0;
 unsigned char *lpbData,*lpbDataOut;
 int	gray,k;
 //lpbData ppV mem bmp brute
 lpbData = &lpMemIn[0];
 lpbDataOut = &lpMemOut[0];
 // pour toutes les donnees
 while(lIndex < lNbrByte) {
	 gray	= lpbData[lIndex];
	 k = 0xff - gray;
	 if (k > 128)
		gray = 0xFF;
	 else
		gray = 0x00;
	lpbDataOut[lIndex] = (BYTE) gray;
	lIndex++;
  	
 }
}
//////////////////////////////////////////////////////////////////////////////////////
///////								GRAY  DITHER
/////// Input parameters
///////				lpMemIn	: gray data (1 byte by dot)
///////				Height	: height of the BMP image (648)
///////				Width	: width of the BMP image (1016)
/////// Output parameter
///////				lpMemOut: pointer on the monochrome data, result of the gray conversion
//////////////////////////////////////////////////////////////////////////////////////
void GrayToDither(unsigned char *lpMemIn ,unsigned char *lpMemOut, long Width, long Height)
{
 long	index_line = 0,index_dot = 0;
 unsigned char *lpbData,*lpbDataOut;
 int	gray,k,*i;
 i = &k;
 lpbData = &lpMemIn[0];
 lpbDataOut = &lpMemOut[0];
 if(Height < Width) {
	index_line = Width;
	Width = Height;
	Height = index_line;
 }

	 for(index_line = 0; index_line < Height; index_line++)
	 {
			for(index_dot = 0 ; index_dot < Width; index_dot++)
			{
				 gray	= lpbData[index_dot + (index_line * Width)];
				 k = 0xff - gray;
				 if (k > DitherPattern1[index_line%8][index_dot%8] )
						gray = 0xFF;
				 else
						gray = 0x00;
				lpbDataOut[index_dot + (index_line * Width)] = (BYTE) gray;
			}
	 }
 
}

//////////////////////////////////////////////////////////////////////////////////////
///////								GRAY  FLOYD
/////// Input parameters
///////				lpMemIn	: gray data (1 byte by dot)
///////				Height	: height of the BMP image (648)
///////				Width	: width of the BMP image (1016)
/////// Output parameter
///////				lpMemOut: pointer on the monochrome data, result of the gray conversion
//////////////////////////////////////////////////////////////////////////////////////
void GrayToFloyd(unsigned char *lpMemIn ,unsigned char *lpMemOut, long Width, long Height)
{
 long	index_line = 0,index_dot = 0;
 unsigned char *lpbData,*lpbDataOut;
 int	gray,k,l;
 int	Error[1032];
 int	NextError[1032];
 int	Cerror;
 int	CE3, CE5, CE7, CE1;

 if(Height < Width) {
	index_line = Width;
	Width = Height;
	Height = index_line;
 }

 for(index_line = 0; index_line < 1032; index_line++) {
		Error[index_line] = NextError[index_line] = 0;
 }
 lpbData = &lpMemIn[0];
 lpbDataOut = &lpMemOut[0];
 for(index_line = 0; index_line < Height; index_line++)
 {
		for(index_dot = 0 ; index_dot < Width; index_dot++)
		{
			gray	= lpbData[index_dot + (index_line * Width)];
			k = (gray + (int) Error[index_dot]);
			if (k > -128 && k < 128)
				l = 0x00;
			else
				l = 0xFF;
			
			lpbDataOut[index_dot + (index_line * Width)] = 0xFF - (BYTE)l;
			
			Cerror = k - l;
			CE3 = (Cerror * 3 / 16);
			CE5 = (Cerror * 5 / 16);
			CE7 = (Cerror * 7 / 16);
			CE1 = Cerror - CE3 - CE5 - CE7;
			NextError[index_dot] += CE5;
			if (index_dot < 1031)
			{
				Error[index_dot+1] += CE7;
				NextError[index_dot+1] += CE1;
			}
			if (index_dot > 0)
				NextError[index_dot-1] += CE3;
		}
		for(index_dot=0; index_dot < 1032; index_dot++)
		{
			Error[index_dot] = NextError[index_dot];
			NextError[index_dot] = 0;
		}	
 }
}

////////////////////////////////////////////////////////////////////////////////////
///////						RVB  to K
/////// Input parameters
///////				lpbRVB	: rvb color data from a 24 bits 1 panel BMP file
///////				RVBSize	: number of byte for the color data [(1016*648)]*3
///////				Height	: height of the BMP image (648)
///////				Width	: width of the BMP image (1016)
///////				Type	: type of monochrome dealing (threshold, floyd or dither)
/////// Output parameter
///////				lpBlack	: pointer on the monochrome data, result of the color conversion
//////////////////////////////////////////////////////////////////////////////////////
BOOL ConvertRVBtoK(unsigned char *lpbRVB,long RVBSize,long Height,long Width,
				   unsigned char *lpBlack,unsigned char type)
{
	long	y = 0,x = 0;
	long	index_line = 0 , index_dot = 0;

	////Conversion Gray to Black
	switch(type) {
	case 't' :
		GrayToThreshold(&lpbRVB[(4*RVBSize)/5],lpBlack,(RVBSize/5));
		break;
	case 'f':
		GrayToFloyd(&lpbRVB[(4*RVBSize)/5],lpBlack, Width, Height);
		break;
	case'd':
		GrayToDither(&lpbRVB[(4*RVBSize)/5],lpBlack, Width, Height);
		break;
	default:
		return(FALSE);
	}

 return(TRUE);
}

//========================================================//
// OPTION : Not real black printed
// Insert sublimable black from K panel to Y,M & C panels
//========================================================//
void KinYMC(unsigned char *lpMem,long Height, long Width)
{
 long	dwSize, i;
 unsigned char *lpYellow,*lpMagenta,*lpCyan,*lpBlack;
 
 dwSize = Width * Height;
 lpYellow  = &lpMem[0]; 
 lpMagenta = &lpMem[dwSize];  
 lpCyan    = &lpMem[2 * dwSize]; 
 lpBlack   = &lpMem[3 * dwSize];
 for(i=0;i < dwSize; i++) {
	 if(lpBlack[i]) {
		lpYellow[i]		|= lpBlack[i];
		lpMagenta[i]	|= lpBlack[i];
		lpCyan[i]		|= lpBlack[i];
	 }
 }
}