/*
    This file is part of the Evolis SDK.

    The Evolis SDK is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The Evolis SDK is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with the Evolis SDK.  If not, see <http://www.gnu.org/licenses/>.
*/

// bmp2prn.cpp : entry point for the console application.
//

#include "stdafx.h"

static const char* version = "1.9.28.2";
// example: -i photo.bmp  -x 600 -y 5  card.prn
void test();

int main(int argc, char* argv[])
{
	char*			inputfile				= NULL;
	char*			outputfile				= NULL;
	int				x, y;
	int				flag					= 0;
	int				ret;
	int				numberOfByteToWrite; 
	unsigned char*	pCard					= NULL;
	char			color[]					= { 'y', 'm', 'c' };
	FILE*			out						= NULL;
	unsigned char*	command					= NULL;
	
	if (parseCommandLine(argc, argv, &inputfile, &outputfile, &x, &y, &flag))
	{
		print_usage();
		exit( ERR_USAGE );
	}

	if (flag & FLAG_CONV)
		ret = OpenBmpEx(inputfile, &pCard, x, y);
	else
		ret = OpenBmp(inputfile, &pCard, x, y);
		
	if (ret)
	{	
		fprintf(stderr, "Warning: OpenBmp failed (exit:%d)\n", ret);
		exit(ret);
	}
	
	// 19 is the printing command length (Dbc) without binary data
	command = new unsigned char[PIX_CARD_SIZE+19];
		
	if (!command) 
	{
		fprintf(stderr, "ERROR: memory allocation failed from main()\r\n");
		delete(pCard);
		exit(ERR_MEM);
	}
	
	out = fopen(outputfile, "w+b");
	if (out==NULL) 
	{
		fprintf(stderr, "ERROR: fopen(%s) failed\r\n", outputfile);
		delete(pCard);
		exit(ERR_IO);
	}
	
	//Download Y, M and C panels
	for(int i = 0; i < 3; i++) 
	{
		numberOfByteToWrite = DB128C(pCard, i*PIX_CARD_SIZE, color[i], command, PIX_CARD_SIZE+19, false, 0, 0);
		if (fwrite(command, sizeof(char), numberOfByteToWrite, out) != numberOfByteToWrite)
		{
			fprintf(stderr, "WARNING: incomplete writing operation\r\n");
		}
	}

	delete (pCard); 	
	delete (command);
	fclose(out);	
	return NO_ERR;
}

int parseCommandLine(int argc, char* argv[], char** pInputfile, char** pOutputfile, int* x, int* y, int *pflag)
{
	int flag = 0;
	char *xvalue = NULL;
	char *yvalue = NULL;
	
	for (int i=1; i<argc; i++)
	{
		if ( (strlen(argv[i])>1) && ((argv[i][0] == '-') || (argv[i][0] == '/')))
		{
			if ((argv[i][1] == 'v') || (argv[i][1] == 'V'))
			{
				printf("%s\r\n", version);
				exit(0);
			}
			else
			if ((argv[i][1] == 'h') || (argv[i][1] == 'H') || (argv[i][1] == '?'))
			{
				print_usage();
				exit(0);
			}
			else
			if ((argv[i][1] == 'i') || (argv[i][1] == 'I'))
			{
				if (i<argc-1) 
				{
					i++;
					*pInputfile = &argv[i][0];
					flag |= FLAG_INPUT;
				}
				else 
					return ERR_USAGE;
			}
			else
			if ((argv[i][1] == 'o') || (argv[i][1] == 'O'))
			{
				if (i<argc-1) 
				{
					i++;
					if (!stricmp(argv[i], "convert")) 
					{
						flag |= FLAG_CONV;
					}
				}
			}
			else
			if ((argv[i][1] == 'x') || (argv[i][1] == 'X'))
			{
				if (i<argc-1) 
				{
					i++;
					*x = atoi(argv[i]);
					flag |= FLAG_POSX;
				}
				else 
					return ERR_USAGE;
			}
			else
			if ((argv[i][1] == 'y') || (argv[i][1] == 'Y'))
			{
				if (i<argc-1) 
				{
					i++;
					*y = atoi(argv[i]);
					flag |= FLAG_POSY;
				}
				else 
					return ERR_USAGE;
			}
		}
		else 
		{
			*pOutputfile = &argv[i][0];
			flag |= FLAG_OUTPUT;
		}
	} // for
	if ((flag & FLAG_GOOD_ARGS) == FLAG_GOOD_ARGS )
	{
		thenth2dot(x);
		thenth2dot(y);
		if ((*y<0) || (*y>647) || (*x<0) || (*x>1016))
		{
			fprintf(stderr, "ERROR: invalid bitmap offset (x:%d y:%d)\r\n", *x, *y);
			return ERR_BMP;
		}
		*pflag = flag;
		return NO_ERR;
	}
	
	fprintf(stderr, "ERROR: missing argument\r\n");
	return ERR_USAGE;
}


void print_usage()
{	
	puts("This program permits to convert a photo bitmap into graphical data readable by an Evolis Printer\r\n");
	puts("Copyright (C) 20019  Evolis Card Printer\r\n");

	puts("This program is free software; you can redistribute it and/or");
	puts("modify it under the terms of the GNU General Public License");
	puts("as published by the Free Software Foundation; either version 2");
	puts("of the License, or (at your option) any later version.\r\n");

	puts("This program is distributed in the hope that it will be useful,");
	puts("but WITHOUT ANY WARRANTY; without even the implied warranty of");
	puts("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the");
	puts("GNU General Public License for more details.\r\n");

	puts("You should have received a copy of the GNU General Public License");
	puts("along with this program; if not, write to the Free Software");
	puts("Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.\r\n");

	
	printf("Version: %s\r\n", version);
	puts("USAGE: bmp2prn [OPTIONS]");
	puts("\t-i filename\t: the input file (uncompress bitmap 24bits)");
	puts("\t-x NNNN\t\t: the offset X");
	puts("\t-y NNNN\t\t: the offset Y");
	puts("\t-v \t\t: the version of this program");
	puts("\t-o convert \t: to convert a image file to bitmap");
	puts("\t-h \t\t: this help");
	puts("\nExample: bmp2prn -i photo.jpeg  -x 600 -y 5 -o convert data.bin");
	fflush(stdout);
}

void thenth2dot(int* x)
{	
	*x = ( (*x) *  118 + 50 ) / 100 ;
}