// This file is part of the Evolis SDK.
//
// The Evolis SDK is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//    
// The Evolis SDK is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with the Evolis SDK.  If not, see <http://www.gnu.org/licenses/>.


// Defines the entry point for the console application.
#include "iomem.h"
#include "ReadMagExample.h"

//
// Timeout 
#define TIMEOUT				5000			// the timeout is 5 seconds. The timeout must be increased for longer command. 
//											// For example, an escape command that involves a printing sequence like 'Se' 
											// after a Db or Dbc command is a long command. Its timeout should be at least 30000.
											// Please notice this timeout is related to printer internal operations and not to communication.

int main(int argc, char* argv[])
{
	char answer[STD_ANSWER_SIZE]	= "";
	char track1[STD_ANSWER_SIZE]	= "";
	char track2[STD_ANSWER_SIZE]	= "";
	char track3[STD_ANSWER_SIZE]	= "";
	int	 iReturnCode;
	
	if (argc!=5) 
	{
		printf("Syntax Error\nUsage : printername track1 track2 track3\n");
		exit(1);
	}

	return ReadAndEncode(argv[1], argv[2], argv[3], argv[4]); 
}

int ReadAndEncode(LPSTR printername, LPSTR outputTrack1, LPSTR outputTrack2, LPSTR outputTrack3)
{
	char answer[STD_ANSWER_SIZE]	= "";
	char track1[STD_ANSWER_SIZE]	= "";
	char track2[STD_ANSWER_SIZE]	= "";
	char track3[STD_ANSWER_SIZE]	= "";
	char cmd[STD_ANSWER_SIZE]		= "";
	int  length;
	int	 iReturnCode;

	// STEP 1: READ MAGNETIC TRACKS
	
	// To set software printer error recovery, 
	if (iReturnCode = SendCommand(printername, "\033Pem;2\015", answer))	return Fail(printername, "Pem;2", iReturnCode);
	if (iReturnCode = SendCommand(printername, "\033Pmi;M;s\015", answer))	return Fail(printername, "Pmi;M;s", iReturnCode);
	// Read all magnetic tracks and save it into a buffer
	if (iReturnCode = SendCommand(printername, "\033Smr\015", answer))		return Fail(printername, "Smr", iReturnCode);
	
	// Copy the content of track1, track2, track3
	if (iReturnCode = SendCommand(printername, "\033Rmb;1\015", track1))	return Fail(printername, "Rmb;1", iReturnCode);
	if (iReturnCode = SendCommand(printername, "\033Rmb;2\015", track2))	return Fail(printername, "Rmb;2", iReturnCode);
	if (iReturnCode = SendCommand(printername, "\033Rmb;3\015", track3))	return Fail(printername, "Rmb;3", iReturnCode);
	
	printf("Track1: %s, Track2: %s, Track3: %s\n", track1, track2, track3);
	puts("Press ENTER to encode (CTRL-C to abort)");
	getchar();

	// STEP 2: WRITE MAGNETIC TRACKS

	// download: ====== TRACK1 ====== 

	length = strlen(outputTrack1);
	if ((length>0) && (length - 7 /* strlen("\033Dm;1;...\015") */ < STD_ANSWER_SIZE))
	{
		strcpy(cmd, "\033Dm;1;");
		strcat(cmd, outputTrack1);
		strcat(cmd, "\015");
		
		if (iReturnCode = SendCommand(printername, cmd, answer))		
			return Fail(printername, cmd, iReturnCode);
	}
	else
	{
		return Fail(printername, "Track1: incorrect length", ERROR_BAD_ARGUMENTS);
	}

	// download: ====== TRACK2 ====== 

	length = strlen(outputTrack2);
	if ((length>0) && (length - 7 /* strlen("\033Dm;1;...\015") */ < STD_ANSWER_SIZE))
	{
		strcpy(cmd, "\033Dm;2;");
		strcat(cmd, outputTrack2);
		strcat(cmd, "\015");
		
		if (iReturnCode = SendCommand(printername, cmd, answer))		
			return Fail(printername, cmd, iReturnCode);
	}
	else
	{
		return Fail(printername, "Track2: incorrect length", ERROR_BAD_ARGUMENTS);
	}

	// download: ====== TRACK3 ====== 

	length = strlen(outputTrack3);
	if ((length>0) && (length - 7 /* strlen("\033Dm;1;...\015") */ < STD_ANSWER_SIZE))
	{
		strcpy(cmd, "\033Dm;3;");
		strcat(cmd, outputTrack3);
		strcat(cmd, "\015");
		
		if (iReturnCode = SendCommand(printername, cmd, answer))		
			return Fail(printername, cmd, iReturnCode);
	}
	else
	{
		return Fail(printername, "Track3: incorrect length", ERROR_BAD_ARGUMENTS);
	}

	// Write tracks
	if (iReturnCode = SendCommand(printername, "\033Smw\015", answer))		Fail(printername, "Smw", iReturnCode);

	// Eject the card
	if (iReturnCode = SendCommand(printername, "\033Se\015", answer))		Fail(printername, "Se", iReturnCode);

	// restore settings
	if (iReturnCode = SendCommand(printername, "\033Pem;0\015", answer))	Fail(printername, "Pem;0", iReturnCode);
	
	puts("Writing done");
	puts("Press ENTER to continue ...\n");
	getchar();
	return 0;
}

int SendCommand(LPSTR printername, LPSTR cmd, LPSTR answer)
{
	DWORD  dwBytesRead = 0;
	BOOL   bRet        = FALSE;
	HANDLE hPrinter	   = NULL;
	int	   iReturnCode = 0;
	
	hPrinter = OpenPebble(printername);
	if (!hPrinter)
	{
		fprintf( stderr, "Error: communication with printer broken\n" );
		return RETURN_ERR_COMM;
	}
	else
	{
		SetTimeout(TIMEOUT);  
		bRet = WritePebble(hPrinter, cmd, (DWORD) strlen(cmd));
		if (bRet) 
		{ 
			bRet = ReadPebble(hPrinter, answer, STD_ANSWER_SIZE, &dwBytesRead);
			if (!bRet) 
			{
				fprintf(stderr, "Error: fail to read data from printer\n");
				iReturnCode = RETURN_ERR_READ;
			}
		}
		else  
		{
			fprintf(stderr, "Error: fail to write data to printer\n");
			iReturnCode = RETURN_ERR_WRITE;
		}
		ClosePebble(hPrinter);
		return RETURN_OK;
	}
}

int Fail(LPSTR printername, LPSTR cmd, int ret)
{
	if (ret & RETURN_ERR_COMM)
	{ 
		fprintf(stderr, "Printer: %s Command: %s Return: %s\n", printername, cmd, "Error Comm.");
	}
	if (ret & RETURN_ERR_WRITE)
	{ 
		fprintf(stderr, "Printer: %s Command: %s Return: %s\n", printername, cmd, "Write failed");
	}
	if (ret & RETURN_ERR_COMM)
	{ 
		fprintf(stderr, "Printer: %s Command: %s Return: %s\n", printername, cmd, "Read failed");
	}	  
	return ret;
}