//-----------------------------------------------------------------------
// <copyright file="Form1.cs" company="Evolis">
//     Evolis Card Printer (c) 2009.
// </copyright>

// This file is part of the Evolis SDK.
// 
// The Evolis SDK is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
// 
// The Evolis SDK is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with the Evolis SDK.  If not, see <http:// www.gnu.org/licenses/>.
// 
// -----------------------------------------------------------------------
namespace TattooRWdirectPrinting
{
   using System;
   using System.Drawing;
   using System.IO;
   using System.Windows.Forms;

   /// <summary>
   /// Main Form: main entry for this program
   /// </summary>
   public partial class Form1 : Form
   {
      /// <summary>
      /// the filename of the black panel bitmap 
      /// </summary>
      private string bitmapKfile;

      /// <summary>
      /// the filename of the overlay panel bitmap 
      /// </summary>
      private string bitmapOfile;

      /// <summary>
      /// Initializes a new instance of the <see cref="Form1"/> class.
      /// </summary>
      public Form1()
      {
         InitializeComponent();
      }
      
      /// <summary>
      /// Check if the bitmap's format is compatible with the card format/
      /// </summary>
      /// <param name="panel">the input bitmap</param>
      /// <returns>true if the bitmap is ok else false</returns>
      private static bool IsValidCardBitmap(Image panel)
      {
         if (panel == null)
         {
            throw new ArgumentNullException("panel");
         }
         
         if ((panel.Height < 1017) && (panel.Width < 649) && (panel.PixelFormat == System.Drawing.Imaging.PixelFormat.Format1bppIndexed))
         {
            return true;
         }
         
         if ((panel.Width < 1017) && (panel.Height < 649) && (panel.PixelFormat == System.Drawing.Imaging.PixelFormat.Format1bppIndexed))
         {
            return true;
         }
         
         return false;
      }
      
      /// <summary>
      /// This button launches the printing sequence
      /// </summary>
      /// <param name="sender">standard object parameter</param>
      /// <param name="e">standard EventArgs parameter</param>
      private void Button3_Click(object sender, EventArgs e)
      {
         if (textBox1.Text == string.Empty) 
         {
            toolStripStatusLabel2.Text = "Please, enter the printer name";
            return;
         }
         
         if (textBox3.Text == string.Empty)
         {
            toolStripStatusLabel2.Text = "Please, select a bitmap";
            return;
         }

         toolStripStatusLabel2.Text = string.Empty;

         bool isSpecificErasing = checkBox1.Checked;
         BinaryReader br;
         FileStream fsK;
         FileStream fsO;
         int pos = 0;
         string cmd1;
         string cmd2;
         string cmd3;

         int printerPtr = Iomem.OpenPebble(textBox1.Text);

         if (printerPtr == 0)
         {
            toolStripStatusLabel2.Text = "Can\'t open printer";
         }
         else
         {
            // Error management to improve ...

            // EVOLIS
            // Please see the programming guide for more information relative to Dbmp command
            // Here the bitmaps for the black or Overlay Panel must be a monochrome 1016x648 bitmap (300dpi)
            // It is possible to use smaller bitmaps. In this case, the position of the bitmap could be adjusted
            // (see Dbmp documentation)

            // cmd header for Black Panel
            cmd1 = Iomem.ESC + "Pem;2" + Iomem.CR +
                   Iomem.ESC + "Pdt;TR" + Iomem.CR +
                   Iomem.ESC + "Prw;mblack" + Iomem.CR +
                   Iomem.ESC + "Pmi;F" + Iomem.CR +
                   Iomem.ESC + "Pme;a" + Iomem.CR +
                   Iomem.ESC + "Pc;k;=;" + trackBar1.Value.ToString() + Iomem.CR +
                   Iomem.ESC + "Pc;e;=;" + trackBar3.Value.ToString() + Iomem.CR +
                   Iomem.ESC + "Ss" + Iomem.CR +
                   Iomem.ESC + "Wcb;k" + Iomem.CR +
                   Iomem.ESC + "Pwb;k" + Iomem.CR +
                   Iomem.ESC + "Pwr;0" + Iomem.CR +
                   Iomem.ESC + "Dbmp;k;0;648;0;";

            // Header for Overlay Panel
            if (isSpecificErasing) 
            {
               cmd2 = Iomem.CRESC + "Wcb;o" + Iomem.CR +
                      Iomem.ESC   + "Pwb;o" + Iomem.CR +
                      Iomem.ESC   + "Dbmp;o;0;648;0;";

               cmd3 = Iomem.CRESC + "Se"    + Iomem.CR +
                      Iomem.ESC   + "Pwb;k" + Iomem.CR +
                      Iomem.ESC   + "Pem;0" + Iomem.CR;
            }
            else
            {
               cmd2 = Iomem.CRESC + "Wcb;o;255" + Iomem.CR;
                      
               cmd3 = Iomem.ESC   + "Se"    + Iomem.CR +
                      Iomem.ESC   + "Pem;0" + Iomem.CR;
            }
            
            // reading of the bmp file sizes
            fsK = File.Open(this.bitmapKfile, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
            int szK = (int)fsK.Length;
            int szO;
            if (isSpecificErasing)
            {
               fsO = File.Open(this.bitmapOfile, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
               szO = (int)fsO.Length;
            }
            else
            {
               fsO = null;
               szO = 0;
            }
            
            byte[] data = new byte[cmd1.Length + cmd2.Length + cmd3.Length + szK + szO + 1];

            System.Text.Encoding.ASCII.GetBytes(cmd1).CopyTo(data, 0);
            pos = pos + cmd1.Length;

            br = new BinaryReader(fsK);
            br.ReadBytes(szK).CopyTo(data, pos);
            pos = pos + szK;
            br.Close();

            System.Text.Encoding.ASCII.GetBytes(cmd2).CopyTo(data, pos);
            pos = pos + cmd2.Length;

            // reading of the bmp file relative to Overlay panel
            if (isSpecificErasing)
            {
               br = new BinaryReader(fsO);
               br.ReadBytes(szO).CopyTo(data, pos);
               pos = pos + szO;
               br.Close();
            }
            
            System.Text.Encoding.ASCII.GetBytes(cmd3).CopyTo(data, pos);

            bool res = Iomem.WritePebble(printerPtr, data, data.Length);

            toolStripStatusLabel2.Text = res ? "Ok" : "Fail";

           Iomem.ClosePebble(printerPtr);
         }
      }

      /// <summary>
      /// Selection of the bitmap relative to the black panel
      /// </summary>
      /// <param name="sender">standard object parameter</param>
      /// <param name="e">standard EventArgs parameter</param>
      private void Button2_Click(object sender, EventArgs e)
      {
         openFileDialog1.InitialDirectory = Application.ExecutablePath;
         openFileDialog1.Filter = "Bitmap File (*.bmp)|*.bmp||";
         openFileDialog1.FilterIndex = 0;
         if (openFileDialog1.ShowDialog() != DialogResult.OK)
         {
               return;
         }
         
         this.bitmapKfile = openFileDialog1.FileName;
         Bitmap kpanel = new Bitmap(this.bitmapKfile);
         if (IsValidCardBitmap(kpanel))
         {
            pictureBox2.Image = kpanel;
            textBox3.Text = openFileDialog1.FileName;
            toolStripStatusLabel2.Text = string.Empty;
         }
         else
         {
            toolStripStatusLabel2.Text = "Error incompatible bitmap format (B/W 1016x648)";
         }
      }

      /// <summary>
      /// Selection of the bitmap relative to the overlay panel
      /// </summary>
      /// <param name="sender">standard object parameter</param>
      /// <param name="e">standard EventArgs parameter</param>
      private void Button1_Click(object sender, EventArgs e)
      {
         openFileDialog1.InitialDirectory = Application.ExecutablePath;
         openFileDialog1.Filter = "Bitmap File (*.bmp)|*.bmp||";
         openFileDialog1.FilterIndex = 0;
         if (openFileDialog1.ShowDialog() != DialogResult.OK)
         {
            return;
         }
         
         this.bitmapOfile = openFileDialog1.FileName;
         Bitmap panelO = new Bitmap(this.bitmapOfile);
         if (IsValidCardBitmap(panelO))
         {
            pictureBox1.Image = panelO;
            textBox2.Text = openFileDialog1.FileName;
            toolStripStatusLabel2.Text = string.Empty;
         }
         else
         {
            toolStripStatusLabel2.Text = "Error incompatible bitmap format (B/W 1016x648)";
         }
      }

      /// <summary>
      /// this checkbox allows the management of the overlay bitmap (erasing bitmap)
      /// </summary>
      /// <param name="sender">standard object parameter</param>
      /// <param name="e">standard EventArgs parameter</param>
      private void CheckBox1_CheckedChanged(object sender, EventArgs e)
      {
         groupBox2.Enabled = checkBox1.Checked;
      }

      /// <summary>
      /// close button
      /// </summary>
      /// <param name="sender">standard object parameter</param>
      /// <param name="e">standard EventArgs parameter</param>
      private void Button4_Click(object sender, EventArgs e)
      {
         this.Close();
      }

      /// <summary>
      /// slide to select printing intensity
      /// </summary>
      /// <param name="sender">standard object parameter</param>
      /// <param name="e">standard EventArgs parameter</param>
      private void TrackBar1_Scroll(object sender, EventArgs e)
      {
         textBox5.Text = trackBar1.Value.ToString();
      }

      /// <summary>
      /// slide to select erasing intensity
      /// </summary>
      /// <param name="sender">standard object parameter</param>
      /// <param name="e">standard EventArgs parameter</param>
      private void TrackBar3_Scroll(object sender, EventArgs e)
      {
         textBox4.Text = trackBar3.Value.ToString();
      }
   }
}