//-----------------------------------------------------------------------
// <copyright file="Iomem.cs" company="Evolis">
//     Evolis Card Printer (c) 2009.
// </copyright>

/// 
/// This file is part of the Evolis SDK.
///
/// The Evolis SDK is free software: you can redistribute it and/or modify
/// it under the terms of the GNU Lesser General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// any later version.
///
/// The Evolis SDK is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU Lesser General Public License for more details.
///
/// You should have received a copy of the GNU Lesser General Public License
/// along with the Evolis SDK.  If not, see <http://www.gnu.org/licenses/>.
///
/// <summary>
///   iomem.dll wrapping class
/// </summary>
namespace TattooRWdirectPrinting
{
   using System;
   using System.Runtime.InteropServices;
   
   /// <summary>
   /// iomem.dll wrapping class
   /// </summary>
   public sealed class Iomem
   {
      /// <summary>
      /// the beginning of an escape command
      /// </summary>
      public const char ESC = '\u001B';
      
      /// <summary>
      /// the end of an escape command
      /// </summary>
      public const char CR = '\r';
      
      /// <summary>
      /// To summerize CR+ESC
      /// </summary>
      public const string CRESC = "\r\u001B"; 
      
      /// <summary>
      /// open a connexion with the printer's port
      /// </summary>
      /// <param name="printerName">the printer name (according to the spooler)</param>
      /// <returns>a handle and NULL if the connexion failed</returns>
      [DllImport("Iomem.dll", EntryPoint = "_OpenPebble@4")]public static extern int OpenPebble([InAttribute()]string printerName);
      
      /// <summary>
      /// close the connexion with the printer's port
      /// </summary>
      /// <param name="printerHandle">the handle provided by OpenPebble</param>
      /// <returns>the status of the operation</returns>
      [DllImport("Iomem.dll", EntryPoint = "_ClosePebble@4")]public static extern bool ClosePebble([InAttribute()]int printerHandle);
      
      /// <summary>
      /// Read data from the printer
      /// </summary>
      /// <param name="printerHandle">the handle provided by OpenPebble</param>
      /// <param name="answer">address to an array</param>
      /// <param name="answerSize">the size of the answer's array</param>
      /// <param name="dataSize">the number of char. need send by the printer</param>
      /// <returns>the status of the operation</returns>
      [DllImport("Iomem.dll", EntryPoint = "_ReadPebble@16")]public static extern bool ReadPebble([InAttribute()]int printerHandle, [In, OutAttribute()]IntPtr answer, [InAttribute()]int answerSize, [In, OutAttribute()]ref int dataSize);
      
      /// <summary>
      /// write data to the printer 
      /// </summary>
      /// <param name="printerHandle">the handle provided by OpenPebble</param>
      /// <param name="cde">data to send to printer</param>
      /// <param name="cdeSize">size of data to send</param>
      /// <returns>the status of the operation</returns>
      [DllImport("Iomem.dll", EntryPoint = "_WritePebble@12", CharSet = CharSet.Ansi)]public static extern bool WritePebble([InAttribute()]int printerHandle, [InAttribute()]byte[] cde, [InAttribute()]int cdeSize);
      
      /// <summary>
      /// get the version
      /// </summary>
      /// <param name="version">address of an integer that contains the iomem's version</param>
      /// <returns>the status of the operation</returns>
      [DllImport("Iomem.dll", EntryPoint = "_GetIomemVersion@4")]public static extern bool GetIomemVersion([OutAttribute()]IntPtr version);
      
      /// <summary>
      /// get the printer internal timeout
      /// </summary>
      /// <returns>the timeout (millisecond)</returns>
      [DllImport("Iomem.dll", EntryPoint = "_GetTimeout@0")]public static extern int GetTimeout();
      
      /// <summary>
      /// set the printer internal timeout
      /// </summary>
      /// <param name="timeout">in millisecond</param>
      /// <returns>the status of the operation</returns>
      [DllImport("Iomem.dll", EntryPoint = "_SetTimeout@4")]public static extern bool SetTimeout([InAttribute()]int timeout);
      
      /// <summary>
      /// get the status of the printer's port (ONLY for USB)
      /// </summary>
      /// <param name="printerHandle">the handle provided by OpenPebble</param>
      /// <param name="statusMask">the port status</param>
      /// <returns>the status of the operation</returns>
      [DllImport("Iomem.dll", EntryPoint = "_GetStatusEvo@8")]public static extern bool GetStatusEvo([InAttribute()]int printerHandle, [In, OutAttribute()]ref int statusMask);
   }
}