/*
    This file is part of the Evolis SDK.

    The Evolis SDK is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The Evolis SDK is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with the Evolis SDK.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "EvoGDI.h"
#include <strsafe.h>

// -------------------------------------------------------------------------------------------

BOOL Evolis_DriverPrint_OpenCard ( PEvolisCard pCard, LPTSTR pPrinterName )
{

#ifdef EVODEBUGLEVEL
	fputs( "Evolis_DriverPrint_OpenCard\r\n", STDEVO );
#endif

	// RZ du status d'erreur
	
	EvoInitError( pCard );
	
	pCard->bNewPenCreated	= FALSE;
	pCard->currPen			= NULL;
	pCard->prevPen			= NULL;
	pCard->bNewFontCreated	= FALSE;
	pCard->currFont			= NULL;
	pCard->prevFont			= NULL;
	pCard->height			= 648;
	pCard->width			= 1016;
	
	// ouverture du printer pour extraction contexte graphique
	
	if ( OpenPrinter( pPrinterName, &(pCard->hPrinter), NULL ) ) 
	{
		LONG lDVsize = DocumentProperties( NULL, pCard->hPrinter, pPrinterName, NULL, NULL, 0 );
		
		if ( lDVsize ) 
		{
			LPBYTE	bInDV	= new BYTE[ lDVsize ];
			LPBYTE  bOutDV	= new BYTE[ lDVsize ];
			DEVMODE	sDV;
			
			if ( ( bInDV != NULL ) && ( bOutDV != NULL ) )
			{
				lDVsize = DocumentProperties( NULL, pCard->hPrinter, pPrinterName, (PDEVMODE) bOutDV, NULL, DM_COPY );

				if (lDVsize == IDOK) 
				{
					if ( IDOK == DocumentProperties(NULL, pCard->hPrinter, pPrinterName, (PDEVMODE)bInDV, (PDEVMODE)bOutDV,
													DM_IN_BUFFER | /*DM_IN_PROMPT |*/ DM_OUT_BUFFER)) 
					{
						memcpy( &sDV, bInDV, sizeof(DEVMODE) );

						if ( pCard->hDC = CreateDC( NULL, pPrinterName, NULL, &sDV ) ) 
						{
							DOCINFO dinfo;
							
							dinfo.cbSize		= sizeof(DOCINFO);
							dinfo.lpszDocName	= NULL;
							dinfo.lpszOutput	= NULL;//"FILE:";
							dinfo.lpszDatatype	= TEXT("raw");
							
							if ( StartDoc( pCard->hDC, &dinfo ) > 0 ) 
							{
								if ( StartPage( pCard->hDC ) <= 0 )
								{
									EvoError( pCard, "Error : StartPage Failed \r\n" );
								}
							} 
							else
							{
								EvoError( pCard, "Error : StartDoc Failed \r\n" );
							}
						}
						else // from CreateDC
						{
							EvoError( pCard , "Error : CreateDC Failed \r\n" );	
						}
					}
					else  // from IDOK == DocumentProperties
					{
						EvoError( pCard, "Error : DocumentProperties Failed \r\n" );
					}
				}
				else // from (lDVsize == IDOK)
				{
					EvoError( pCard, "Error : DocumentProperties (Copy Request) Failed \r\n" );		
				}
			}
			else  // from ( ( bInDV != NULL ) && ( bOutDV != NULL ) )
			{
				EvoError( pCard, "Error : Memory Allocation Failed \r\n" );
			}
			
			// nettoyage 
			delete(bInDV);
			delete(bOutDV);
			if ( !ClosePrinter(pCard->hPrinter) ) 
			{
				EvoError( pCard, "Error : ClosePrinter Failed \r\n" );
			}
		}
		else // from lDVSize
		{
			EvoError( pCard, "Error : DocumentProperties (Size Request) Failed \r\n" );
		}
	} 
	else // openprinter
	{
		EvoError( pCard, "Error : OpenPrinter Failed \r\n" );
	}
	
	return ( !pCard->bOnError );
}

// -------------------------------------------------------------------------------------------

BOOL Evolis_DriverPrint_PrintCard ( PEvolisCard pCard )
{

#ifdef EVODEBUGLEVEL
	fputs( "Evolis_DriverPrint_PrintCard\r\n", STDEVO );
#endif

	// test the previous status 
	
	if ( pCard->bOnError ) // Can't print in this case
	{
		EvoError( pCard, "Error status detected. Please try to re-open.\r\n" );
	}
		
	if ( EndPage( pCard->hDC ) <= 0 ) 
	{
		EvoError( pCard, "Error : EndPage Failed \r\n" );
	}
	
	if ( EndDoc( pCard->hDC ) <= 0 )
	{
		EvoError( pCard, "Error : EndDoc Failed \r\n" );
	}
	if ( ! DeleteDC( pCard->hDC ) )
	{
		EvoError( pCard, "Error : DeleteDC Failed \r\n" );
	}
	
	if ( pCard->bNewFontCreated )
	{
		if ( DeleteObject(pCard->currFont) )	
		{
#if (EVODEBUGLEVEL > 1)
		fputs( "Evolis_DriverPrint_PrintCard - Font deleted\r\n", STDEVO );
#endif	
		}
		else
		{
			EvoError( pCard, "Error : DeleteObject (Font) Failed \r\n" );
		}
		
		pCard->bNewFontCreated	= FALSE;
		pCard->currFont			= NULL;
		pCard->prevFont			= NULL;
		
	} // from ( pCard->bNewFontCreated )
	
	if ( pCard->bNewPenCreated )
	{
		if ( DeleteObject(pCard->currPen) )	
		{
#if (EVODEBUGLEVEL > 1)
		fputs( "Evolis_DriverPrint_PrintCard - Pen deleted\r\n", STDEVO );
#endif	
		}
		else
		{
			EvoError( pCard, "Error : DeleteObject (Pen) Failed \r\n" );
		}
		
		pCard->bNewPenCreated	= FALSE;
		pCard->currPen			= NULL;
		pCard->prevPen			= NULL;
		
	} // from ( pCard->bNewPenCreated )
	
	return ( !pCard->bOnError );
}

// -------------------------------------------------------------------------------------------

BOOL Evolis_DriverPrint_AddBitmap ( PEvolisCard pCard, LPTSTR bmpFile, int nXOffset, int nYOffset, int nWidth, int nHeight)
{
	HBITMAP hBmp;

#ifdef EVODEBUGLEVEL
	fputs( "Evolis_DriverPrint_AddBitmap\r\n", STDEVO );
#endif
	
	if ( pCard->bOnError ) 
	{
		EvoError( pCard, "On Error. Please re-open.\r\n" );
		return FALSE;
	}

	hBmp = (HBITMAP) LoadImage( NULL, bmpFile, IMAGE_BITMAP, 
							   0,0, LR_DEFAULTSIZE | LR_DEFAULTCOLOR | LR_LOADFROMFILE );

	if ( hBmp )
	{
		SetMapMode( pCard->hDC, MM_TEXT );
		if ( ! SetBitMapPixels( &(pCard->hDC), hBmp, 0, nXOffset, nYOffset, nWidth, nHeight ) )
		{
			EvoError( pCard, "Error : SetBitMapPixels failed" );
		}
		DeleteObject( hBmp ); 
	} 
	else //  from hBmp
	{
		EvoError( pCard, "Error : LoadBitmap Failed\r\n" );
	}
	
	return ( !pCard->bOnError );
}

// -------------------------------------------------------------------------------------------

BOOL Evolis_DriverPrint_AddText ( PEvolisCard pCard, LPTSTR strText, int nTopX, int nTopY )
{
	size_t szlgth; 
	
#ifdef EVODEBUGLEVEL
	fputs( "Evolis_DriverPrint_AddText\r\n", STDEVO );
#endif
	
	if ( pCard->bOnError ) 
	{
		EvoError( pCard, "On Error. Please re-open.\r\n" );
		return FALSE;
	}
	
	StringCchLength ( strText, EVO_STRING_LIMIT, &szlgth );
	if ( ! TextOut( pCard->hDC, nTopX, nTopY, strText, (int) szlgth ) )
	{
		EvoError( pCard, "Error : TextOut Failed\r\n" );
	}
	return ( !pCard->bOnError );
}

// -------------------------------------------------------------------------------------------

BOOL Evolis_DriverPrint_AddFormatedText ( PEvolisCard pCard, LPTSTR strText, int nFormat, int nTopX, int nTopY, int nBottomX, int nBottomY  )
{
	size_t	szlgth;
	RECT	rect; 
		
#ifdef EVODEBUGLEVEL
	fputs( "Evolis_DriverPrint_AddFormatedText\r\n", STDEVO );
#endif
	
	if ( pCard->bOnError ) 
	{
		EvoError( pCard, "On Error. Please re-open.\r\n" );
		return FALSE;
	}
	
	rect.bottom = nBottomY;
	rect.left	= nTopX;
	rect.right  = nBottomX;
	rect.top	= nTopY;

	StringCchLength ( strText, EVO_STRING_LIMIT, &szlgth );

	if ( ! DrawText( pCard->hDC, strText, (int) szlgth, &rect, nFormat ) )
	{
		EvoError( pCard, "Error : TextOut Failed\r\n" );
	}

	return ( !pCard->bOnError );
}

// -------------------------------------------------------------------------------------------

BOOL Evolis_DriverPrint_SetColors ( PEvolisCard pCard, COLORREF fgColor, COLORREF bgColor )
{
#ifdef EVODEBUGLEVEL
	fputs( "Evolis_DriverPrint_SetColors\r\n", STDEVO );
#endif
	
	if ( pCard->bOnError ) 
	{
		EvoError( pCard, "On Error. Please re-open.\r\n" );
		return FALSE;
	}
	
	if ( SetBkColor( pCard->hDC, bgColor ) != CLR_INVALID )
	{
		if ( SetTextColor( pCard->hDC, fgColor ) == CLR_INVALID )
		{
			EvoError( pCard, "Error : SetTextColor failed\r\n" );
			return FALSE;
		}	
	} 
	else 
	{
		EvoError( pCard, "Error : SetBkColor failed\r\n" );
		return FALSE;
	}
	return TRUE;
}

// -------------------------------------------------------------------------------------------

BOOL Evolis_DriverPrint_SetTextAttrib ( PEvolisCard pCard, int height, int width, int angle, int weight, int isUnderline, int isItalic, LPCTSTR lpszFace )
{
	
#ifdef EVODEBUGLEVEL
	fputs( "Evolis_DriverPrint_SetTextAttrib\r\n", STDEVO );
#endif
	
	if ( pCard->bOnError ) 
	{
		EvoError( pCard, "On Error. Please re-open.\r\n" );
		return FALSE;
	}
	
	if ( pCard->bNewFontCreated )
	{
		SelectObject( pCard->hDC, pCard->prevFont );
		
		if ( DeleteObject(pCard->currFont) )	
		{
#if (EVODEBUGLEVEL > 1)
		fputs( "Evolis_DriverPrint_SetTextAttrib - Font deleted\r\n", STDEVO );
#endif	
		}
		else
		{
			EvoError( pCard, "Error : DeleteObject Failed (Evolis_DriverPrint_SetTextAttrib)\r\n" );
		}
	} // from ( pCard->bNewFontCreated )
	
	pCard->currFont = CreateFont(
						height,							// height of font
						width,							// average character width
						10*angle,						// angle of escapement
						10*angle,						// base-line orientation angle
						weight,							// font weight
						isItalic,						// italic attribute option
						isUnderline,					// underline attribute option
						EVO_DEFAULT_ATTRIB,				// strikeout attribute option
						EVO_DEFAULT_ATTRIB,				// character set identifier
						EVO_DEFAULT_ATTRIB,				// output precision
						EVO_DEFAULT_ATTRIB,				// clipping precision
						EVO_DEFAULT_ATTRIB,				// output quality
						EVO_DEFAULT_ATTRIB,				// pitch and family
						lpszFace           				// typeface name
					);

	if ( pCard->currFont != NULL )
	{
		// Select our rotated font structure and save previous font info
		pCard->prevFont			= (HFONT) SelectObject( pCard->hDC, pCard->currFont );
		pCard->bNewFontCreated	= TRUE;
	}
	else
	{	
		EvoError( pCard, "Error : CreateFont Failed\r\n" );
	}

	return ( !pCard->bOnError );
}

// -------------------------------------------------------------------------------------------

BOOL Evolis_DriverPrint_SetTextAttribEx ( PEvolisCard pCard, int height, int width, int angle, int weight, int isUnderline, int isItalic, LPCTSTR lpszFace )
{
	
#ifdef EVODEBUGLEVEL
	fputs( "Evolis_DriverPrint_SetTextAttrib\r\n", STDEVO );
#endif
	
	if ( pCard->bOnError ) 
	{
		EvoError( pCard, "On Error. Please re-open.\r\n" );
		return FALSE;
	}
	
	if ( pCard->bNewFontCreated )
	{
		SelectObject( pCard->hDC, pCard->prevFont );
		
		if ( DeleteObject(pCard->currFont) )	
		{
#if (EVODEBUGLEVEL > 1)
		fputs( "Evolis_DriverPrint_SetTextAttrib - Font deleted\r\n", STDEVO );
#endif	
		}
		else
		{
			EvoError( pCard, "Error : DeleteObject Failed (Evolis_DriverPrint_SetTextAttrib)\r\n" );
		}
	} // from ( pCard->bNewFontCreated )
	
	pCard->currFont = CreateFont(
						height,							// height of font
						width,							// average character width
						10*angle,						// angle of escapement
						10*angle,						// base-line orientation angle
						weight,							// font weight
						isItalic,						// italic attribute option
						isUnderline,					// underline attribute option
						EVO_DEFAULT_ATTRIB,				// strikeout attribute option
						EVO_DEFAULT_ATTRIB,				// character set identifier
						EVO_DEFAULT_ATTRIB,				// output precision
						EVO_DEFAULT_ATTRIB,				// clipping precision
						EVO_DEFAULT_ATTRIB,				// output quality
						EVO_DEFAULT_ATTRIB,				// pitch and family
						lpszFace           				// typeface name
					);

	if ( pCard->currFont != NULL )
	{
		// Select our rotated font structure and save previous font info
		pCard->prevFont			= (HFONT) SelectObject( pCard->hDC, pCard->currFont );
		pCard->bNewFontCreated	= TRUE;
	}
	else
	{	
		EvoError( pCard, "Error : CreateFont Failed\r\n" );
	}

	return ( !pCard->bOnError );
}

// -------------------------------------------------------------------------------------------
int Evolis_DriverPrint_MagEncode ( PEvolisCard pCard, LPTSTR strDataTrack1, LPTSTR strDataTrack2, LPTSTR strDataTrack3 )
{
	
	char lpstrcmd[EVO_STRING_LIMIT] = "00";   // Important
	size_t szSize, sz1, sz2, sz3;
	BOOL bDataToSend = FALSE;
	
#ifdef EVODEBUGLEVEL
	fputs( "Evolis_DriverPrint_MagEncode\r\n", STDEVO );
#endif
	
	if ( pCard->bOnError ) 
	{
		EvoError( pCard, "On Error. Please re-open.\r\n" );
		return FALSE;
	}
	
	StringCchLength(strDataTrack1, EVO_STRING_LIMIT, &sz1 );
	StringCchLength(strDataTrack2, EVO_STRING_LIMIT, &sz2 );
	StringCchLength(strDataTrack3, EVO_STRING_LIMIT, &sz3 );
	
	if ( sz1 > 0 )
	{
		StringCchCat(lpstrcmd, EVO_STRING_LIMIT, TEXT("\033Dm;1;"));
		StringCchCat(lpstrcmd, EVO_STRING_LIMIT, strDataTrack1);
		StringCchCat(lpstrcmd, EVO_STRING_LIMIT, TEXT("\015") );
		bDataToSend = TRUE;
	}
	if ( sz2 > 0 )
	{
		StringCchCat(lpstrcmd, EVO_STRING_LIMIT, TEXT("\033Dm;2;"));
		StringCchCat(lpstrcmd, EVO_STRING_LIMIT, strDataTrack2);
		StringCchCat(lpstrcmd, EVO_STRING_LIMIT, TEXT("\015") );
		bDataToSend = TRUE;
	}
	if ( sz3 > 0 )
	{
		StringCchCat(lpstrcmd, EVO_STRING_LIMIT, TEXT("\033Dm;3;"));
		StringCchCat(lpstrcmd, EVO_STRING_LIMIT, strDataTrack3);
		StringCchCat(lpstrcmd, EVO_STRING_LIMIT, TEXT("\015"));
		bDataToSend = TRUE;
	}	

	// PASSTHROUGH length parameter (16bits)

	StringCchLength(lpstrcmd, EVO_STRING_LIMIT, &szSize );

	lpstrcmd[0] = (char) (szSize % 256);
	lpstrcmd[1] = (char) (szSize / 256);

	// Send command to printer
	if (bDataToSend)
	{
		return Escape( pCard->hDC, PASSTHROUGH, 0, lpstrcmd, NULL);
	}
	else
	{	
		return 0;
	}
	// the return is described at http://msdn.microsoft.com/en-us/library/ms535472(VS.85).aspx
}

// -------------------------------------------------------------------------------------------

BOOL Evolis_DriverPrint_AddLine ( PEvolisCard pCard, int nStartX, int nStartY, int nEndX, int nEndY )
{
#ifdef EVODEBUGLEVEL
	fputs( "Evolis_DriverPrint_AddLine\r\n", STDEVO );
#endif
	
	if ( pCard->bOnError ) 
	{
		EvoError( pCard, "On Error. Please re-open.\r\n" );
		return FALSE;
	}
	
	if ( MoveToEx( pCard->hDC, nStartX, nStartY, NULL ) )
	{
		 if ( !LineTo( pCard->hDC, nEndX, nEndY ) )
		 {
			EvoError( pCard, "Error : LineTo Failed\r\n" );
		 }
	}
	else 
	{
		EvoError( pCard, "Error : MoveToEx Failed\r\n" );
	}
	return (! pCard->bOnError );
}


// -------------------------------------------------------------------------------------------

BOOL Evolis_DriverPrint_AddEllipse( PEvolisCard pCard, COLORREF fillColor, int nStartX, int nStartY, int nEndX, int nEndY )
{
	
	HBRUSH hbrush, hbrushOld; 

#ifdef EVODEBUGLEVEL
	fputs( "Evolis_DriverPrint_AddEllipse\r\n", STDEVO );
#endif
	
	if ( pCard->bOnError ) 
	{
		EvoError( pCard, "On Error. Please re-open.\r\n" );
		return FALSE;
	}

	hbrush = CreateSolidBrush( fillColor );
	
	if ( !hbrush ) 
	{
		EvoError( pCard, "Error : CreateSolidBrush Failed\r\n" );
		return FALSE;
	}
	
	hbrushOld = (HBRUSH) SelectObject( pCard->hDC, hbrush );
	
	if ( !Ellipse( pCard->hDC, nStartX, nStartY, nEndX, nEndY ) )
	{
			EvoError( pCard, "Error : Ellipse Failed\r\n" );
	}
	
	SelectObject( pCard->hDC, hbrushOld );
    DeleteObject( hbrush );
	
	return (! pCard->bOnError );
}

// -------------------------------------------------------------------------------------------

BOOL Evolis_DriverPrint_AddRectangle( PEvolisCard pCard, COLORREF fillColor, int nStartX, int nStartY, int nEndX, int nEndY )
{
	HBRUSH hbrush, hbrushOld; 

#ifdef EVODEBUGLEVEL
	fputs( "Evolis_DriverPrint_AddEllipse\r\n", STDEVO );
#endif
	
	if ( pCard->bOnError ) 
	{
		EvoError( pCard, "On Error. Please re-open.\r\n" );
		return FALSE;
	}

	hbrush = (HBRUSH) CreateSolidBrush( fillColor );
	
	if ( !hbrush ) 
	{
		EvoError( pCard, "Error : CreateSolidBrush Failed\r\n" );
		return FALSE;
	}
	
	hbrushOld = (HBRUSH) SelectObject( pCard->hDC, hbrush );
	
	if ( !Rectangle( pCard->hDC, nStartX, nStartY, nEndX, nEndY ) )
	{
			EvoError( pCard, "Error : Ellipse Failed\r\n" );
	}
	
	SelectObject( pCard->hDC, hbrushOld );
    DeleteObject( hbrush );
	
	return (! pCard->bOnError );
}

// -------------------------------------------------------------------------------------------

BOOL Evolis_DriverPrint_SetLineAttrib( PEvolisCard pCard, int nThickness, COLORREF fgColor )
{
   
#ifdef EVODEBUGLEVEL
	fputs( "Evolis_DriverPrint_SetLineAttrib\r\n", STDEVO );
#endif
	
	if ( pCard->bOnError ) 
	{
		EvoError( pCard, "On Error. Please re-open.\r\n" );
		return FALSE;
	}

	if ( pCard->bNewPenCreated )
	{
		SelectObject( pCard->hDC, pCard->prevPen );
		
		if ( DeleteObject(pCard->currPen) )	
		{
#if (EVODEBUGLEVEL > 1)
		fputs( "Evolis_DriverPrint_SetLineAttrib - Pen deleted\r\n", STDEVO );
#endif	
		}
		else
		{
			EvoError( pCard, "Error : DeleteObject Failed (Evolis_DriverPrint_SetLineAttrib)\r\n" );
		}
	} // from ( pCard->bNewPenCreated )

	pCard->currPen = CreatePen(PS_SOLID, nThickness, fgColor);
	
	if ( ! pCard->currPen ) 
	{
		EvoError( pCard, "Error : CreatePen Failed\r\n" );
		return FALSE;
	}
	
	pCard->prevPen = (HPEN) SelectObject( pCard->hDC, pCard->currPen );
	
	pCard->bNewPenCreated = TRUE;
	
	return (! pCard->bOnError );
}

// -------------------------------------------------------------------------------------------

// return TRUE if success 
//		  FALSE else

int SetBitMapPixels(HDC *phDC, HBITMAP hBmp, COLORREF rgbTransparent, int nXOffset, int nYOffset, int iW, int iH)
{
	//Sanity..
	if (hBmp    == NULL || phDC     == NULL)
		return FALSE;
	
	HDC		hMemDC = CreateCompatibleDC(NULL);
	
	if (hMemDC)
	{
		// Get bitmap size
		BITMAP bm;
		GetObject(hBmp, sizeof(bm), &bm);

		//Make sure x and Y offset are relative
		//to center of the bitmap; this
		//way the bitmap will appear centered
		//on (nXOffset,nYOffset)
		//nXOffset -= bm.bmWidth /2;
		//nYOffset -= bm.bmHeight/2;

		// Create a 32 bits depth bitmap and select it into the memory DC
		BITMAPINFOHEADER RGB32BITSBITMAPINFO =
		{	
			sizeof(BITMAPINFOHEADER),// biSize
			bm.bmWidth,				 // biWidth;
           -bm.bmHeight,			 // biHeight
			1,						 // biPlanes;
			32,						 // biBitCount
			BI_RGB,					 // biCompression;
			0,						 // biSizeImage;
			0,						 // biXPelsPerMeter;
			0,						 // biYPelsPerMeter;
			0,						 // biClrUsed;
			0						 // biClrImportant;
		};

		VOID * pbits32; 

		HBITMAP hbm32 = CreateDIBSection(hMemDC,(BITMAPINFO *)&RGB32BITSBITMAPINFO,DIB_RGB_COLORS,&pbits32,NULL,0);

		if (hbm32)
		{
			HBITMAP holdBmp = (HBITMAP)SelectObject(hMemDC, hbm32);
			// Create a DC just to copy the bitmap into the memory D
			HDC hDC = CreateCompatibleDC(hMemDC);
			if (hDC)
			{
				// Get how many bytes per row w
				//have for the bitmap bit
				//(rounded up to 32 bits
				BITMAP bm32;
				HBITMAP holdBmp;
				GetObject(hbm32, sizeof(bm32), &bm32);

				while (bm32.bmWidthBytes % 4)
					bm32.bmWidthBytes++;

				// Copy the bitmap into the memory D
				holdBmp = (HBITMAP)SelectObject(hDC, hBmp);
				BitBlt(hMemDC,0,0,bm.bmWidth,bm.bmHeight,hDC,0,0,SRCCOPY);

				SelectObject(hDC, holdBmp);
				DeleteDC(hDC);
			}
			StretchBlt(*phDC,nXOffset,nYOffset,iW,iH,hMemDC,0,0,bm.bmWidth,bm.bmHeight,SRCCOPY);
			DeleteObject(SelectObject(hMemDC, holdBmp));
		}
		else
		{
			DeleteDC(hMemDC);
			return FALSE;
		}
		DeleteDC(hMemDC);
	}
	else return FALSE;

	return TRUE;
}

DWORD EvoError( PEvolisCard pCard, char* errorMsg )
{
	pCard->bOnError		= TRUE;
	pCard->dwLastError	= GetLastError();

#if (EVODEBUGLEVEL > 1)
	fputs( errorMsg, STDEVO );
	fprintf( STDEVO, "\t Last Error : %lu\r\n", pCard->dwLastError );	
#endif

#if (EVODEBUGLEVEL > 2)
	EvoCardPrintInfo( pCard );	
#endif		

	return pCard->dwLastError;						
}

DWORD EvoInitError( PEvolisCard pCard )
{
	pCard->bOnError = FALSE;
	return	( pCard->dwLastError = ERROR_SUCCESS ) ;
}

void EvoCardPrintInfo( PEvolisCard pCard ) 
{
#if (EVODEBUGLEVEL>2)
	fprintf( STDEVO, "\t\t --------\r\n \t\t %lX\r\n\t\t %lX\r\n\t\t %d\r\n\t\t %d\r\n\t\t --------\r\n ", 
					(DWORD)pCard->hDC, (DWORD)pCard->hPrinter, pCard->height, pCard->width );
#endif					
}
